@@ -12,7 +12,7 @@ The interface consists of the following functions:
   default_parameters - Default parameter values for FFC
 """
 
-__version__ = "1.4.0"
+__version__ = "1.6.0"
 
 # Import compiler functions
 from ffc.compiler import compile_form, compile_element
@@ -21,24 +21,24 @@ from ffc.compiler import compile_form, compile_element
 from ffc.jitcompiler import jit
 
 # Import default parameters
-from parameters import default_parameters
+from .parameters import default_parameters
 
 # Import plotting
-from plot import *
+from .plot import *
 
 # Import useful extra functionality
-from extras import *
+from .extras import *
 
 # List of supported elements
 try:
 
     # Import list of supported elements from FIAT
     from FIAT import supported_elements
-    supported_elements = supported_elements.keys()
+    supported_elements = list(supported_elements.keys())
     supported_elements.sort()
 
     # Append elements that we can plot
-    from plot import element_colors
+    from .plot import element_colors
     supported_elements_for_plotting = list(set(supported_elements).union(set(element_colors.keys())))
     supported_elements_for_plotting.sort()
 
