@@ -1,7 +1,7 @@
-# Code generation format strings for UFC (Unified Form-assembly Code) v. 1.4.0.
+# Code generation format strings for UFC (Unified Form-assembly Code) v. 1.6.0.
 # This code is released into the public domain.
 #
-# The FEniCS Project (http://www.fenicsproject.org/) 2006-2014.
+# The FEniCS Project (http://www.fenicsproject.org/) 2006-2015.
 
 form_combined = """\
 /// This class defines the interface for the assembly of the global
@@ -41,6 +41,7 @@ public:
 %(signature)s
   }
 
+
   /// Return the rank of the global tensor (r)
   virtual std::size_t rank() const
   {
@@ -53,36 +54,57 @@ public:
 %(num_coefficients)s
   }
 
-  /// Return the number of cell domains
-  virtual std::size_t num_cell_domains() const
+  /// Return original coefficient position for each coefficient (0 <= i < n)
+  virtual std::size_t original_coefficient_position(std::size_t i) const
   {
-%(num_cell_domains)s
+%(original_coefficient_position)s
+  }
+
+
+  /// Create a new finite element for argument function i
+  virtual ufc::finite_element* create_finite_element(std::size_t i) const
+  {
+%(create_finite_element)s
+  }
+
+  /// Create a new dofmap for argument function i
+  virtual ufc::dofmap* create_dofmap(std::size_t i) const
+  {
+%(create_dofmap)s
+  }
+
+
+  /// Return the number of cell domains
+  virtual std::size_t max_cell_subdomain_id() const
+  {
+%(max_cell_subdomain_id)s
   }
 
   /// Return the number of exterior facet domains
-  virtual std::size_t num_exterior_facet_domains() const
+  virtual std::size_t max_exterior_facet_subdomain_id() const
   {
-%(num_exterior_facet_domains)s
+%(max_exterior_facet_subdomain_id)s
   }
 
   /// Return the number of interior facet domains
-  virtual std::size_t num_interior_facet_domains() const
+  virtual std::size_t max_interior_facet_subdomain_id() const
   {
-%(num_interior_facet_domains)s
+%(max_interior_facet_subdomain_id)s
   }
 
-  /// Return the number of point domains
-  virtual std::size_t num_point_domains() const
+  /// Return the number of vertex domains
+  virtual std::size_t max_vertex_subdomain_id() const
   {
-%(num_point_domains)s
+%(max_vertex_subdomain_id)s
   }
 
   /// Return the number of custom domains
-  virtual std::size_t num_custom_domains() const
+  virtual std::size_t max_custom_subdomain_id() const
   {
-%(num_custom_domains)s
+%(max_custom_subdomain_id)s
   }
 
+
   /// Return whether the form has any cell integrals
   virtual bool has_cell_integrals() const
   {
@@ -101,10 +123,10 @@ public:
 %(has_interior_facet_integrals)s
   }
 
-  /// Return whether the form has any point integrals
-  virtual bool has_point_integrals() const
+  /// Return whether the form has any vertex integrals
+  virtual bool has_vertex_integrals() const
   {
-%(has_point_integrals)s
+%(has_vertex_integrals)s
   }
 
   /// Return whether the form has any custom integrals
@@ -113,48 +135,38 @@ public:
 %(has_custom_integrals)s
   }
 
-  /// Create a new finite element for argument function i
-  virtual ufc::finite_element* create_finite_element(std::size_t i) const
-  {
-%(create_finite_element)s
-  }
 
-  /// Create a new dofmap for argument function i
-  virtual ufc::dofmap* create_dofmap(std::size_t i) const
-  {
-%(create_dofmap)s
-  }
-
-  /// Create a new cell integral on sub domain i
-  virtual ufc::cell_integral* create_cell_integral(std::size_t i) const
+  /// Create a new cell integral on sub domain subdomain_id
+  virtual ufc::cell_integral* create_cell_integral(std::size_t subdomain_id) const
   {
 %(create_cell_integral)s
   }
 
-  /// Create a new exterior facet integral on sub domain i
-  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(std::size_t i) const
+  /// Create a new exterior facet integral on sub domain subdomain_id
+  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(std::size_t subdomain_id) const
   {
 %(create_exterior_facet_integral)s
   }
 
-  /// Create a new interior facet integral on sub domain i
-  virtual ufc::interior_facet_integral* create_interior_facet_integral(std::size_t i) const
+  /// Create a new interior facet integral on sub domain subdomain_id
+  virtual ufc::interior_facet_integral* create_interior_facet_integral(std::size_t subdomain_id) const
   {
 %(create_interior_facet_integral)s
   }
 
-  /// Create a new point integral on sub domain i
-  virtual ufc::point_integral* create_point_integral(std::size_t i) const
+  /// Create a new vertex integral on sub domain subdomain_id
+  virtual ufc::vertex_integral* create_vertex_integral(std::size_t subdomain_id) const
   {
-%(create_point_integral)s
+%(create_vertex_integral)s
   }
 
-  /// Create a new custom integral on sub domain i
-  virtual ufc::custom_integral* create_custom_integral(std::size_t i) const
+  /// Create a new custom integral on sub domain subdomain_id
+  virtual ufc::custom_integral* create_custom_integral(std::size_t subdomain_id) const
   {
 %(create_custom_integral)s
   }
 
+
   /// Create a new cell integral on everywhere else
   virtual ufc::cell_integral* create_default_cell_integral() const
   {
@@ -173,10 +185,10 @@ public:
 %(create_default_interior_facet_integral)s
   }
 
-  /// Create a new point integral on everywhere else
-  virtual ufc::point_integral* create_default_point_integral() const
+  /// Create a new vertex integral on everywhere else
+  virtual ufc::vertex_integral* create_default_vertex_integral() const
   {
-%(create_default_point_integral)s
+%(create_default_vertex_integral)s
   }
 
   /// Create a new custom integral on everywhere else
@@ -217,26 +229,39 @@ public:
   /// Return a string identifying the form
   virtual const char* signature() const;
 
+
   /// Return the rank of the global tensor (r)
   virtual std::size_t rank() const;
 
   /// Return the number of coefficients (n)
   virtual std::size_t num_coefficients() const;
 
+  /// Return original coefficient position for each coefficient (0 <= i < n)
+  virtual std::size_t original_coefficient_position(std::size_t i) const;
+
+
+  /// Create a new finite element for argument function i
+  virtual ufc::finite_element* create_finite_element(std::size_t i) const;
+
+  /// Create a new dofmap for argument function i
+  virtual ufc::dofmap* create_dofmap(std::size_t i) const;
+
+
   /// Return the number of cell domains
-  virtual std::size_t num_cell_domains() const;
+  virtual std::size_t max_cell_subdomain_id() const;
 
   /// Return the number of exterior facet domains
-  virtual std::size_t num_exterior_facet_domains() const;
+  virtual std::size_t max_exterior_facet_subdomain_id() const;
 
   /// Return the number of interior facet domains
-  virtual std::size_t num_interior_facet_domains() const;
+  virtual std::size_t max_interior_facet_subdomain_id() const;
 
-  /// Return the number of point domains
-  virtual std::size_t num_point_domains() const;
+  /// Return the number of vertex domains
+  virtual std::size_t max_vertex_subdomain_id() const;
 
   /// Return the number of custom domains
-  virtual std::size_t num_custom_domains() const;
+  virtual std::size_t max_custom_subdomain_id() const;
+
 
   /// Return whether the form has any cell integrals
   virtual bool has_cell_integrals() const;
@@ -247,17 +272,12 @@ public:
   /// Return whether the form has any interior facet integrals
   virtual bool has_interior_facet_integrals() const;
 
-  /// Return whether the form has any point integrals
-  virtual bool has_point_integrals() const;
+  /// Return whether the form has any vertex integrals
+  virtual bool has_vertex_integrals() const;
 
   /// Return whether the form has any custom integrals
   virtual bool has_custom_integrals() const;
 
-  /// Create a new finite element for argument function i
-  virtual ufc::finite_element* create_finite_element(std::size_t i) const;
-
-  /// Create a new dofmap for argument function i
-  virtual ufc::dofmap* create_dofmap(std::size_t i) const;
 
   /// Create a new cell integral on sub domain i
   virtual ufc::cell_integral* create_cell_integral(std::size_t i) const;
@@ -268,12 +288,13 @@ public:
   /// Create a new interior facet integral on sub domain i
   virtual ufc::interior_facet_integral* create_interior_facet_integral(std::size_t i) const;
 
-  /// Create a new point integral on sub domain i
-  virtual ufc::point_integral* create_point_integral(std::size_t i) const;
+  /// Create a new vertex integral on sub domain i
+  virtual ufc::vertex_integral* create_vertex_integral(std::size_t i) const;
 
   /// Create a new custom integral on sub domain i
   virtual ufc::custom_integral* create_custom_integral(std::size_t i) const;
 
+
   /// Create a new cell integral on everywhere else
   virtual ufc::cell_integral* create_default_cell_integral() const;
 
@@ -283,8 +304,8 @@ public:
   /// Create a new interior facet integral on everywhere else
   virtual ufc::interior_facet_integral* create_default_interior_facet_integral() const;
 
-  /// Create a new point integral on everywhere else
-  virtual ufc::point_integral* create_default_point_integral() const;
+  /// Create a new vertex integral on everywhere else
+  virtual ufc::vertex_integral* create_default_vertex_integral() const;
 
   /// Create a new custom integral on everywhere else
   virtual ufc::custom_integral* create_default_custom_integral() const;
@@ -310,6 +331,7 @@ const char* %(classname)s::signature() const
 %(signature)s
 }
 
+
 /// Return the rank of the global tensor (r)
 std::size_t %(classname)s::rank() const
 {
@@ -322,36 +344,57 @@ std::size_t %(classname)s::num_coefficients() const
 %(num_coefficients)s
 }
 
-/// Return the number of cell domains
-std::size_t %(classname)s::num_cell_domains() const
+/// Return original coefficient position for each coefficient (0 <= i < n)
+std::size_t %(classname)s::original_coefficient_position(std::size_t i) const
 {
-%(num_cell_domains)s
+%(original_coefficient_position)s
+}
+
+
+/// Create a new finite element for argument function i
+ufc::finite_element* %(classname)s::create_finite_element(std::size_t i) const
+{
+%(create_finite_element)s
+}
+
+/// Create a new dofmap for argument function i
+ufc::dofmap* %(classname)s::create_dofmap(std::size_t i) const
+{
+%(create_dofmap)s
+}
+
+
+/// Return the number of cell domains
+std::size_t %(classname)s::max_cell_subdomain_id() const
+{
+%(max_cell_subdomain_id)s
 }
 
 /// Return the number of exterior facet domains
-std::size_t %(classname)s::num_exterior_facet_domains() const
+std::size_t %(classname)s::max_exterior_facet_subdomain_id() const
 {
-%(num_exterior_facet_domains)s
+%(max_exterior_facet_subdomain_id)s
 }
 
 /// Return the number of interior facet domains
-std::size_t %(classname)s::num_interior_facet_domains() const
+std::size_t %(classname)s::max_interior_facet_subdomain_id() const
 {
-%(num_interior_facet_domains)s
+%(max_interior_facet_subdomain_id)s
 }
 
-/// Return the number of point domains
-std::size_t %(classname)s::num_point_domains() const
+/// Return the number of vertex domains
+std::size_t %(classname)s::max_vertex_subdomain_id() const
 {
-%(num_point_domains)s
+%(max_vertex_subdomain_id)s
 }
 
 /// Return the number of custom domains
-std::size_t %(classname)s::num_custom_domains() const
+std::size_t %(classname)s::max_custom_subdomain_id() const
 {
-%(num_custom_domains)s
+%(max_custom_subdomain_id)s
 }
 
+
 /// Return whether the form has any cell integrals
 bool %(classname)s::has_cell_integrals() const
 {
@@ -370,10 +413,10 @@ bool %(classname)s::has_interior_facet_integrals() const
 %(has_interior_facet_integrals)s
 }
 
-/// Return whether the form has any point integrals
-bool %(classname)s::has_point_integrals() const
+/// Return whether the form has any vertex integrals
+bool %(classname)s::has_vertex_integrals() const
 {
-%(has_point_integrals)s
+%(has_vertex_integrals)s
 }
 
 /// Return whether the form has any custom integrals
@@ -382,48 +425,38 @@ bool %(classname)s::has_custom_integrals() const
 %(has_custom_integrals)s
 }
 
-/// Create a new finite element for argument function i
-ufc::finite_element* %(classname)s::create_finite_element(std::size_t i) const
-{
-%(create_finite_element)s
-}
 
-/// Create a new dofmap for argument function i
-ufc::dofmap* %(classname)s::create_dofmap(std::size_t i) const
-{
-%(create_dofmap)s
-}
-
-/// Create a new cell integral on sub domain i
-ufc::cell_integral* %(classname)s::create_cell_integral(std::size_t i) const
+/// Create a new cell integral on sub domain subdomain_id
+ufc::cell_integral* %(classname)s::create_cell_integral(std::size_t subdomain_id) const
 {
 %(create_cell_integral)s
 }
 
-/// Create a new exterior facet integral on sub domain i
-ufc::exterior_facet_integral* %(classname)s::create_exterior_facet_integral(std::size_t i) const
+/// Create a new exterior facet integral on sub domain subdomain_id
+ufc::exterior_facet_integral* %(classname)s::create_exterior_facet_integral(std::size_t subdomain_id) const
 {
 %(create_exterior_facet_integral)s
 }
 
-/// Create a new interior facet integral on sub domain i
-ufc::interior_facet_integral* %(classname)s::create_interior_facet_integral(std::size_t i) const
+/// Create a new interior facet integral on sub domain subdomain_id
+ufc::interior_facet_integral* %(classname)s::create_interior_facet_integral(std::size_t subdomain_id) const
 {
 %(create_interior_facet_integral)s
 }
 
-/// Create a new point integral on sub domain i
-ufc::point_integral* %(classname)s::create_point_integral(std::size_t i) const
+/// Create a new vertex integral on sub domain subdomain_id
+ufc::vertex_integral* %(classname)s::create_vertex_integral(std::size_t subdomain_id) const
 {
-%(create_point_integral)s
+%(create_vertex_integral)s
 }
 
-/// Create a new custom integral on sub domain i
-ufc::custom_integral* %(classname)s::create_custom_integral(std::size_t i) const
+/// Create a new custom integral on sub domain subdomain_id
+ufc::custom_integral* %(classname)s::create_custom_integral(std::size_t subdomain_id) const
 {
 %(create_custom_integral)s
 }
 
+
 /// Create a new cell integral on everywhere else
 ufc::cell_integral* %(classname)s::create_default_cell_integral() const
 {
@@ -442,10 +475,10 @@ ufc::interior_facet_integral* %(classname)s::create_default_interior_facet_integ
 %(create_default_interior_facet_integral)s
 }
 
-/// Create a new point integral on everywhere else
-ufc::point_integral* %(classname)s::create_default_point_integral() const
+/// Create a new vertex integral on everywhere else
+ufc::vertex_integral* %(classname)s::create_default_vertex_integral() const
 {
-%(create_default_point_integral)s
+%(create_default_vertex_integral)s
 }
 
 /// Create a new custom integral on everywhere else
