@@ -6,7 +6,7 @@ This module implements the generation of C++ code for the body of each
 UFC function from an (optimized) intermediate representation (OIR).
 """
 
-# Copyright (C) 2009-2013 Anders Logg
+# Copyright (C) 2009-2015 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -24,10 +24,7 @@ UFC function from an (optimized) intermediate representation (OIR).
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Mehdi Nikbakht 2010
-# Modified by Martin Alnaes, 2013
-#
-# First added:  2009-12-16
-# Last changed: 2014-03-19
+# Modified by Martin Alnaes, 2013-2015
 
 # FFC modules
 from ffc.log import info, begin, end, debug_code
@@ -41,8 +38,7 @@ from ffc.evaluatebasisderivatives import _evaluate_basis_derivatives_all
 from ffc.evaluatedof import evaluate_dof_and_dofs, affine_weights
 from ffc.interpolatevertexvalues import interpolate_vertex_values
 
-from ffc.representation import pick_representation
-
+from ffc.representation import pick_representation, ufc_integral_types
 
 # Errors issued for non-implemented functions
 def _not_implemented(function_name, return_null=False):
@@ -138,8 +134,7 @@ def _generate_element_code(ir, prefix, parameters):
         = _not_implemented("map_from_reference_cell")
     code["map_to_reference_cell"] = _not_implemented("map_to_reference_cell")
     code["num_sub_elements"] = ret(ir["num_sub_elements"])
-    code["create_sub_element"] = _create_foo(prefix, "finite_element",
-                                             ir["create_sub_element"])
+    code["create_sub_element"] = _create_sub_element(prefix, ir)
     code["create"] = ret(create(code["classname"]))
 
     # Postprocess code
@@ -180,7 +175,7 @@ def _generate_dofmap_code(ir, prefix, parameters):
     code["topological_dimension"] = ret(ir["topological_dimension"])
     code["geometric_dimension"] = ret(ir["geometric_dimension"])
     code["global_dimension"] = _global_dimension(ir["global_dimension"])
-    code["local_dimension"] = ret(ir["local_dimension"])
+    code["num_element_dofs"] = ret(ir["num_element_dofs"])
     code["num_facet_dofs"] = ret(ir["num_facet_dofs"])
     code["num_entity_dofs"] \
         = switch(f_d, [ret(num) for num in ir["num_entity_dofs"]],
@@ -193,8 +188,7 @@ def _generate_dofmap_code(ir, prefix, parameters):
     code["tabulate_coordinates"] \
         = _tabulate_coordinates(ir["tabulate_coordinates"])
     code["num_sub_dofmaps"] = ret(ir["num_sub_dofmaps"])
-    code["create_sub_dofmap"] = _create_foo(prefix, "dofmap",
-                                            ir["create_sub_dofmap"])
+    code["create_sub_dofmap"] = _create_sub_dofmap(prefix, ir)
     code["create"] = ret(create(code["classname"]))
 
     # Postprocess code
@@ -202,7 +196,6 @@ def _generate_dofmap_code(ir, prefix, parameters):
 
     return code
 
-
 def _generate_integral_code(ir, prefix, parameters):
     "Generate code for integrals from intermediate representation."
 
@@ -259,26 +252,18 @@ def _generate_form_code(ir, prefix, parameters):
     code["destructor"] = do_nothing
 
     code["signature"] = ret('"%s"' % ir["signature"])
-    code["original_coefficient_position"] \
-        = _generate_original_coefficient_position(ir["original_coefficient_positions"])
+    code["original_coefficient_position"] = _generate_original_coefficient_position(ir["original_coefficient_positions"])
     code["rank"] = ret(ir["rank"])
     code["num_coefficients"] = ret(ir["num_coefficients"])
 
-    code["create_finite_element"] = _create_foo(prefix, "finite_element",
-                                                ir["create_finite_element"])
-    code["create_dofmap"] = _create_foo(prefix, "dofmap", ir["create_dofmap"])
+    code["create_finite_element"] = _create_finite_element(prefix, ir)
+    code["create_dofmap"] = _create_dofmap(prefix, ir)
 
-    integral_types = ["cell", "exterior_facet", "interior_facet", "point",
-                      "custom"]
-    for integral_type in integral_types:
-        code["num_%s_domains" % integral_type] = ret(ir["num_%s_domains"
-                                                        % integral_type])
-        code["has_%s_integrals" % integral_type] \
-            = _has_foo_integrals(ir, integral_type)
-        code["create_%s_integral" % integral_type] \
-            = _create_foo_integral(ir, integral_type, prefix)
-        code["create_default_%s_integral" % integral_type] \
-            = _create_default_foo_integral(ir, integral_type, prefix)
+    for integral_type in ufc_integral_types:
+        code["max_%s_subdomain_id" % integral_type] = ret(ir["max_%s_subdomain_id" % integral_type])
+        code["has_%s_integrals" % integral_type] = _has_foo_integrals(ir, integral_type)
+        code["create_%s_integral" % integral_type] = _create_foo_integral(ir, integral_type, prefix)
+        code["create_default_%s_integral" % integral_type] = _create_default_foo_integral(ir, integral_type, prefix)
 
     # Postprocess code
     _postprocess_code(code, parameters)
@@ -287,7 +272,6 @@ def _generate_form_code(ir, prefix, parameters):
 
 #--- Code generation for non-trivial functions ---
 
-
 def _value_dimension(ir):
     "Generate code for value_dimension."
     ret = format["return"]
@@ -500,9 +484,8 @@ def _tabulate_entity_dofs(ir):
 #--- Utility functions ---
 
 
-def _create_foo(prefix, class_name, postfix, numbers=None):
+def _create_foo(prefix, class_name, postfix, arg, numbers=None):
     "Generate code for create_<foo>."
-    f_i = format["argument sub"]
     ret = format["return"]
     create = format["create foo"]
 
@@ -510,15 +493,30 @@ def _create_foo(prefix, class_name, postfix, numbers=None):
                    for i in postfix]
     cases = [ret(create(name)) for name in class_names]
     default = ret(0)
-    return format["switch"](f_i, cases, default=default, numbers=numbers)
+    return format["switch"](arg, cases, default=default, numbers=numbers)
 
+def _create_finite_element(prefix, ir):
+    f_i = format["argument sub"]
+    return _create_foo(prefix, "finite_element", ir["create_finite_element"], f_i)
+
+def _create_dofmap(prefix, ir):
+    f_i = format["argument sub"]
+    return _create_foo(prefix, "dofmap", ir["create_dofmap"], f_i)
+
+def _create_sub_element(prefix, ir):
+    f_i = format["argument sub"]
+    return _create_foo(prefix, "finite_element", ir["create_sub_element"], f_i)
+
+def _create_sub_dofmap(prefix, ir):
+    f_i = format["argument sub"]
+    return _create_foo(prefix, "dofmap", ir["create_sub_dofmap"], f_i)
 
 def _create_foo_integral(ir, integral_type, prefix):
     "Generate code for create_<foo>_integral."
+    f_i = format["argument subdomain"]
     class_name = integral_type + "_integral_" + str(ir["id"])
     postfix = ir["create_" + integral_type + "_integral"]
-    return _create_foo(prefix, class_name, postfix, numbers=postfix)
-
+    return _create_foo(prefix, class_name, postfix, f_i, numbers=postfix)
 
 def _has_foo_integrals(ir, integral_type):
     ret = format["return"]
@@ -526,7 +524,6 @@ def _has_foo_integrals(ir, integral_type):
     i = ir["has_%s_integrals" % integral_type]
     return ret(b(i))
 
-
 def _create_default_foo_integral(ir, integral_type, prefix):
     "Generate code for create_default_<foo>_integral."
     ret = format["return"]
@@ -551,7 +548,7 @@ def _indent_code(code):
     for key in code:
         if not key in ("classname", "members", "constructor_arguments",
                        "initializer_list", "additional_includes_set",
-                       "restrict"):
+                       "restrict", "class_type"):
             code[key] = indent(code[key], 4)
 
 
