@@ -89,7 +89,7 @@ The compiler stages are implemented by the following functions:
   format_code       (stage 5)
 """
 
-# Copyright (C) 2007-2013 Anders Logg
+# Copyright (C) 2007-2015 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -109,15 +109,13 @@ The compiler stages are implemented by the following functions:
 # Modified by Kristian B. Oelgaard, 2010.
 # Modified by Dag Lindbo, 2008.
 # Modified by Garth N. Wells, 2009.
-# Modified by Martin Alnaes, 2013
-#
-# First added:  2007-02-05
-# Last changed: 2013-01-25
+# Modified by Martin Alnaes, 2013-2015
 
 __all__ = ["compile_form", "compile_element"]
 
 # Python modules
 from time import time
+import os
 
 # FFC modules
 from ffc.log import info, info_green, warning
@@ -128,11 +126,10 @@ from ffc.analysis import analyze_forms, analyze_elements
 from ffc.representation import compute_ir
 from ffc.optimization import optimize_ir
 from ffc.codegeneration import generate_code
-from ffc.formatting import format_code
+from ffc.formatting import format_code, write_code
 from ffc.wrappers import generate_wrapper_code
 
-def compile_form(forms, object_names={}, prefix="Form",\
-                 parameters=default_parameters()):
+def compile_form(forms, object_names=None, prefix="Form", parameters=None):
     """This function generates UFC code for a given UFL form or list
     of UFL forms."""
 
@@ -143,8 +140,14 @@ def compile_form(forms, object_names={}, prefix="Form",\
 
     # Check input arguments
     forms = _check_forms(forms)
+    if not forms:
+        return
+    if prefix != os.path.basename(prefix):
+        prefix = os.path.basename(prefix)
+        warning("Invalid prefix, modified to {}.".format(prefix))
+    if object_names is None:
+        object_names = {}
     parameters = _check_parameters(parameters)
-    if not forms: return
 
     # Stage 1: analysis
     cpu_time = time()
@@ -173,12 +176,13 @@ def compile_form(forms, object_names={}, prefix="Form",\
 
     # Stage 5: format code
     cpu_time = time()
-    format_code(code, wrapper_code, prefix, parameters)
+    code_h, code_c = format_code(code, wrapper_code, prefix, parameters)
+    write_code(code_h, code_c, prefix, parameters) # FIXME: Don't write to file in this function (issue #72)
     _print_timing(5, time() - cpu_time)
 
     info_green("FFC finished in %g seconds.", time() - cpu_time_0)
 
-def compile_element(elements, prefix="Element", parameters=default_parameters()):
+def compile_element(elements, prefix="Element", parameters=None):
     """This function generates UFC code for a given UFL element or
     list of UFL elements."""
 
@@ -189,8 +193,9 @@ def compile_element(elements, prefix="Element", parameters=default_parameters())
 
     # Check input arguments
     elements = _check_elements(elements)
+    if not elements:
+        return
     parameters = _check_parameters(parameters)
-    if not elements: return
 
     # Stage 1: analysis
     cpu_time = time()
@@ -220,7 +225,8 @@ def compile_element(elements, prefix="Element", parameters=default_parameters())
 
     # Stage 5: format code
     cpu_time = time()
-    format_code(code, wrapper_code, prefix, parameters)
+    code_h, code_c = format_code(code, wrapper_code, prefix, parameters)
+    write_code(code_h, code_c, prefix, parameters) # FIXME: Don't write to file in this function (issue #72)
     _print_timing(5, time() - cpu_time)
 
     info_green("FFC finished in %g seconds.", time() - cpu_time_0)
@@ -239,6 +245,8 @@ def _check_elements(elements):
 
 def _check_parameters(parameters):
     "Initial check of parameters."
+    if parameters is None:
+        parameters = default_parameters()
     if "blas" in parameters:
         warning("BLAS mode unavailable (will return in a future version).")
     if "quadrature_points" in parameters:
