@@ -9,7 +9,7 @@ It relies on templates for UFC code available as part of the module
 ufc_utils.
 """
 
-# Copyright (C) 2009-2014 Anders Logg
+# Copyright (C) 2009-2015 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -25,21 +25,21 @@ ufc_utils.
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
-#
-# First added:  2009-12-16
-# Last changed: 2014-03-19
 
 # Python modules
 import os
 
 # FFC modules
 from ffc.log import info, error, begin, end, dstr
-from ffc.constants import FFC_VERSION, UFC_VERSION
+from ffc import __version__ as FFC_VERSION
+from ffc.backends.ufc import __version__ as UFC_VERSION
 from ffc.cpp import format
 from ffc.backends.ufc import templates
+from ffc.parameters import compilation_relevant_parameters
+
 
 def format_code(code, wrapper_code, prefix, parameters):
-    "Format given code in UFC format."
+    "Format given code in UFC format. Returns two strings with header and source file contents."
 
     begin("Compiler stage 5: Formatting code")
 
@@ -60,44 +60,24 @@ def format_code(code, wrapper_code, prefix, parameters):
     code_c += format["header_c"] % {"prefix": prefix}
 
     # Generate code for elements
-    if code_elements:
-        for code_element in code_elements:
-            code_h += _format_h("finite_element", code_element, parameters)
-            code_c += _format_c("finite_element", code_element, parameters)
+    for code_element in code_elements:
+        code_h += _format_h("finite_element", code_element, parameters)
+        code_c += _format_c("finite_element", code_element, parameters)
 
     # Generate code for dofmaps
-    if code_dofmaps:
-        for code_dofmap in code_dofmaps:
-            code_h += _format_h("dofmap", code_dofmap, parameters)
-            code_c += _format_c("dofmap", code_dofmap, parameters)
+    for code_dofmap in code_dofmaps:
+        code_h += _format_h("dofmap", code_dofmap, parameters)
+        code_c += _format_c("dofmap", code_dofmap, parameters)
 
     # Generate code for integrals
-    if code_integrals:
-        for code_integral in code_integrals:
-            classname = code_integral["classname"]
-            if "cell_integral" in classname:
-                code_h += _format_h("cell_integral", code_integral, parameters)
-                code_c += _format_c("cell_integral", code_integral, parameters)
-            elif "exterior_facet_integral" in classname:
-                code_h += _format_h("exterior_facet_integral", code_integral, parameters)
-                code_c += _format_c("exterior_facet_integral", code_integral, parameters)
-            elif "interior_facet_integral" in classname:
-                code_h += _format_h("interior_facet_integral", code_integral, parameters)
-                code_c += _format_c("interior_facet_integral", code_integral, parameters)
-            elif "point_integral" in classname:
-                code_h += _format_h("point_integral", code_integral, parameters)
-                code_c += _format_c("point_integral", code_integral, parameters)
-            elif "custom_integral" in classname:
-                code_h += _format_h("custom_integral", code_integral, parameters)
-                code_c += _format_c("custom_integral", code_integral, parameters)
-            else:
-                error("Unable to figure out base class for %s" % classname)
+    for code_integral in code_integrals:
+        code_h += _format_h(code_integral["class_type"], code_integral, parameters)
+        code_c += _format_c(code_integral["class_type"], code_integral, parameters)
 
     # Generate code for form
-    if code_forms:
-        for code_form in code_forms:
-            code_h += _format_h("form", code_form, parameters)
-            code_c += _format_c("form", code_form, parameters)
+    for code_form in code_forms:
+        code_h += _format_h("form", code_form, parameters)
+        code_c += _format_c("form", code_form, parameters)
 
     # Add wrappers
     if wrapper_code:
@@ -106,6 +86,11 @@ def format_code(code, wrapper_code, prefix, parameters):
     # Generate code for footer
     code_h += format["footer"]
 
+    end()
+
+    return code_h, code_c
+
+def write_code(code_h, code_c, prefix, parameters):
     # Write file(s)
     if parameters["split"]:
         _write_file(code_h, prefix, ".h", parameters)
@@ -113,8 +98,6 @@ def format_code(code, wrapper_code, prefix, parameters):
     else:
         _write_file(code_h, prefix, ".h", parameters)
 
-    end()
-
 def _format_h(class_type, code, parameters):
     "Format header code for given class type."
     if parameters["split"]:
@@ -131,17 +114,17 @@ def _format_c(class_type, code, parameters):
 
 def _write_file(output, prefix, postfix, parameters):
     "Write generated code to file."
-    prefix = prefix.split(os.path.join(' ',' ').split()[0])[-1]
-    full_prefix = os.path.join(parameters["output_dir"], prefix)
-    filename = "%s%s" % (full_prefix, postfix)
-    hfile = open(filename, "w")
-    hfile.write(output)
-    hfile.close()
+    filename = os.path.join(parameters["output_dir"], prefix + postfix)
+    with open(filename, "w") as hfile:
+        hfile.write(output)
     info("Output written to " + filename + ".")
 
 def _generate_comment(parameters):
     "Generate code for comment on top of file."
 
+    # Drop irrelevant parameters
+    parameters = compilation_relevant_parameters(parameters)
+
     # Generate top level comment
     args = {"ffc_version": FFC_VERSION, "ufc_version": UFC_VERSION}
     if parameters["format"] == "ufc":
@@ -168,4 +151,3 @@ def _generate_additional_includes(codes):
     if s:
         return "\n".join(list(s)) + "\n"
     return ""
-
