@@ -59,7 +59,9 @@ def _optimize_integral_ir(ir, parameters):
     # Select representation
     r = pick_representation(ir["representation"])
 
-    # Optimize representation
-    oir = r.optimize_integral_ir(ir, parameters)
-
-    return oir
+    # Optimize representation (if available for representation)
+    try:
+        oir = r.optimize_integral_ir(ir, parameters)
+        return oir
+    except:
+        return ir
