@@ -11,11 +11,11 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-07-12
 # Last changed: 2010-02-09
@@ -25,13 +25,13 @@ from ffc.log import error
 from ffc.cpp import format
 
 # FFC quadrature modules.
-from symbolics import CONST
+from .symbolics import CONST
 #from symbolics import format
-from symbolics import create_float
-from symbolics import create_product
-from symbolics import create_sum
-from symbolics import create_fraction
-from expr import Expr
+from .symbolics import create_float
+from .symbolics import create_product
+from .symbolics import create_sum
+from .symbolics import create_fraction
+from .expr import Expr
 
 class FloatValue(Expr):
     def __init__(self, value):
@@ -100,7 +100,7 @@ class FloatValue(Expr):
             return create_float(self.val*other.val)
         return other.__mul__(self)
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         "Division by other objects."
         # If division is illegal (this should definitely not happen).
         if other.val == 0.0:
@@ -150,10 +150,12 @@ class FloatValue(Expr):
 
         # Nothing left to do.
         return create_fraction(self, other)
+        
+    __div__ = __truediv__
 
 # FFC quadrature modules.
-from symbol     import Symbol
-from product    import Product
-from sumobj    import Sum
-from fraction   import Fraction
+from .symbol     import Symbol
+from .product    import Product
+from .sumobj    import Sum
+from .fraction   import Fraction
 
