@@ -18,15 +18,19 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009
-#
-# First added:  2009-03-18
-# Last changed: 2014-04-23
 
 # Python modules.
 from numpy import shape
 
+from six import iteritems, iterkeys
+from six.moves import xrange as range
+from six import advance_iterator as next
+def firstkey(d):
+    return next(iterkeys(d))
+
 # UFL common.
 from ufl.common import product
+from ufl.utils.sorting import sorted_by_key
 
 # UFL Classes.
 from ufl.classes import FixedIndex
@@ -35,9 +39,6 @@ from ufl.classes import FloatValue
 from ufl.classes import Coefficient
 from ufl.classes import Operator
 
-# UFL Algorithms.
-from ufl.algorithms.printing import tree_format
-
 # FFC modules.
 from ffc.log import info, debug, error, ffc_assert
 from ffc.cpp import format
@@ -45,7 +46,7 @@ from ffc.quadrature.quadraturetransformerbase import QuadratureTransformerBase
 from ffc.quadrature.quadratureutils import create_permutations
 
 # Symbolics functions
-#from symbolics import set_format
+#from ffc.quadrature.symbolics import set_format
 from ffc.quadrature.symbolics import create_float, create_symbol, create_product,\
                                      create_sum, create_fraction, BASIS, IP, GEO, CONST
 
@@ -72,14 +73,14 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         for op in operands:
             # If entries does already exist we can add the code, otherwise just
             # dump them in the element tensor.
-            for key, val in op.items():
+            for key, val in sorted(op.items()):
                 if key in code:
                     code[key].append(val)
                 else:
                     code[key] = [val]
 
         # Add sums and group if necessary.
-        for key, val in code.items():
+        for key, val in sorted_by_key(code):
             if len(val) > 1:
                 code[key] = create_sum(val)
             elif val:
@@ -103,9 +104,9 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
             # If we get an empty dict, something was zero and so is the product.
             if not op:
                 return {}
-            if len(op) > 1 or (op and op.keys()[0] != ()):
+            if len(op) > 1 or (op and firstkey(op) != ()):
                 permute.append(op)
-            elif op and op.keys()[0] == ():
+            elif op and firstkey(op) == ():
                 not_permute.append(op[()])
 
         # Create permutations.
@@ -121,8 +122,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         if permutations:
             for key, val in permutations.items():
                 # Sort key in order to create a unique key.
-                l = list(key)
-                l.sort()
+                l = sorted(key)
 
                 # TODO: I think this check can be removed for speed since we
                 # just have a list of objects we should never get any conflicts here.
@@ -157,7 +157,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         #print("\n\nVisiting Power: " + repr(o))
 
         # Get base and exponent.
-        base, expo = o.operands()
+        base, expo = o.ufl_operands
 
         # Visit base to get base code.
         base_code = self.visit(base)
@@ -196,6 +196,24 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         new_val = create_symbol(format["absolute value"](str(val)), val.t, val, 1)
         return {():new_val}
 
+    def min_value(self, o, *operands):
+        # Take minimum value of operands.
+        val0 = operands[0][()]
+        val1 = operands[1][()]
+        t = min(val0.t, val1.t)
+        # FIXME: I don't know how to implement this the optimized way. Is this right?
+        new_val = create_symbol(format["min value"](str(val0), str(val1)), t)
+        return {():new_val}
+
+    def max_value(self, o, *operands):
+        # Take maximum value of operands.
+        val0 = operands[0][()]
+        val1 = operands[1][()]
+        t = min(val0.t, val1.t)
+        # FIXME: I don't know how to implement this the optimized way. Is this right?
+        new_val = create_symbol(format["max value"](str(val0), str(val1)), t)
+        return {():new_val}
+
     # -------------------------------------------------------------------------
     # Condition, Conditional (conditional.py).
     # -------------------------------------------------------------------------
@@ -204,7 +222,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         # Get condition expression and do safety checks.
         # Might be a bit too strict?
         c, = operands
-        ffc_assert(len(c) == 1 and c.keys()[0] == (),\
+        ffc_assert(len(c) == 1 and firstkey(c) == (),\
             "Condition for NotCondition should only be one function: " + repr(c))
         sym = create_symbol(format["not"](str(c[()])), c[()].t, base_op=c[()].ops()+1)
         return {(): sym}
@@ -214,9 +232,9 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         # Get LHS and RHS expressions and do safety checks.
         # Might be a bit too strict?
         lhs, rhs = operands
-        ffc_assert(len(lhs) == 1 and lhs.keys()[0] == (),\
+        ffc_assert(len(lhs) == 1 and firstkey(lhs) == (),\
             "LHS of Condtion should only be one function: " + repr(lhs))
-        ffc_assert(len(rhs) == 1 and rhs.keys()[0] == (),\
+        ffc_assert(len(rhs) == 1 and firstkey(rhs) == (),\
             "RHS of Condtion should only be one function: " + repr(rhs))
 
         # Map names from UFL to cpp.py.
@@ -233,14 +251,13 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         return {(): sym}
 
     def conditional(self, o, *operands):
-
         # Get condition and return values; and do safety check.
         cond, true, false = operands
-        ffc_assert(len(cond) == 1 and cond.keys()[0] == (),\
+        ffc_assert(len(cond) == 1 and firstkey(cond) == (),\
             "Condtion should only be one function: " + repr(cond))
-        ffc_assert(len(true) == 1 and true.keys()[0] == (),\
+        ffc_assert(len(true) == 1 and firstkey(true) == (),\
             "True value of Condtional should only be one function: " + repr(true))
-        ffc_assert(len(false) == 1 and false.keys()[0] == (),\
+        ffc_assert(len(false) == 1 and firstkey(false) == (),\
             "False value of Condtional should only be one function: " + repr(false))
 
         # Get values and test for None
@@ -454,7 +471,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
                         code[mapping].append(self.__apply_transform(basis, derivatives, multi, tdim, gdim))
 
         # Add sums and group if necessary.
-        for key, val in code.items():
+        for key, val in list(code.items()):
             if len(val) > 1:
                 code[key] = create_sum(val)
             else:
@@ -565,7 +582,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
                    "MathFunctions expect one operand of function type: " + repr(operands))
         # Use format function on value of operand.
         operand = operands[0]
-        for key, val in operand.items():
+        for key, val in list(operand.items()):
             new_val = create_symbol(format_function(str(val)), val.t, val, 1)
             operand[key] = new_val
         #raise Exception("pause")
@@ -608,7 +625,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         weight = self._create_symbol(weight, ACCESS)[()]
 
         # Create value.
-        if integral_type in ("point", "custom"):
+        if integral_type in ("vertex", "custom"):
             trans_set = set()
             value = create_product([val, weight])
         else:
@@ -619,7 +636,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         # Update sets of used variables (if they will not be used because of
         # optimisations later, they will be reset).
-        trans_set.update(map(lambda x: str(x), value.get_unique_vars(GEO)))
+        trans_set.update([str(x) for x in value.get_unique_vars(GEO)])
         used_points = set([self.points])
         ops = self._count_operations(value)
         used_psi_tables = set([self.psi_tables_map[b]
