@@ -11,11 +11,11 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-07-12
 # Last changed: 2010-03-11
@@ -25,11 +25,12 @@ from ffc.log import error
 from ffc.cpp import format
 
 # FFC quadrature modules.
-from symbolics import create_float
-from symbolics import create_product
-from symbolics import create_sum
-from symbolics import create_fraction
-from expr import Expr
+from .symbolics import create_float
+from .symbolics import create_product
+from .symbolics import create_sum
+from .symbolics import create_fraction
+from .expr import Expr
+from functools import reduce
 
 #class Product(object):
 class Product(Expr):
@@ -204,7 +205,7 @@ class Product(Expr):
         # Create new product adding all variables from other Product.
         return create_product(self.vrs + other.vrs)
 
-    def __div__(self, other):
+    def __truediv__(self, other):
         "Division by other objects."
         # If division is illegal (this should definitely not happen).
         if other.val == 0.0:
@@ -267,6 +268,8 @@ class Product(Expr):
         # If we no longer have a denominater, just return the numerator.
         return num
 
+    __div__ = __truediv__
+
     # Public functions.
     def expand(self):
         "Expand all members of the product."
@@ -404,8 +407,8 @@ class Product(Expr):
         return [(found, remains)]
 
 # FFC quadrature modules.
-from floatvalue import FloatValue
-from symbol     import Symbol
-from sumobj    import Sum
-from fraction   import Fraction
+from .floatvalue import FloatValue
+from .symbol     import Symbol
+from .sumobj    import Sum
+from .fraction   import Fraction
 
