@@ -1,6 +1,6 @@
 "Code generator for quadrature representation."
 
-# Copyright (C) 2009-2013 Kristian B. Oelgaard
+# Copyright (C) 2009-2014 Kristian B. Oelgaard
 #
 # This file is part of FFC.
 #
@@ -19,27 +19,24 @@
 #
 # Modified by Mehdi Nikbakht 2010
 # Modified by Anders Logg 2013-2014
-# Modified by Martin Alnaes 2013
-#
-# First added:  2009-01-07
-# Last changed: 2014-04-23
+# Modified by Martin Alnaes 2013-2014
 
 # Python modules
 import functools, itertools
 import numpy
 
 # UFL modules
-from ufl.algorithms.printing import tree_format
+from ufl.utils.sorting import sorted_by_key
+from ufl.utils.derivativetuples import compute_derivative_tuples
 
 # FFC modules
 from ffc.log import info, debug, ffc_assert, error, warning
 from ffc.cpp import format, remove_unused
-from ffc.utils import compute_derivative_tuples
 
 from ffc.representationutils import initialize_integral_code
 
 # Utility and optimization functions for quadraturegenerator
-from symbolics import generate_aux_constants
+from ffc.quadrature.symbolics import generate_aux_constants
 
 def generate_integral_code(ir, prefix, parameters):
     "Generate code for integral from intermediate representation."
@@ -71,7 +68,7 @@ def _tabulate_tensor(ir, prefix, parameters):
     f_int          = format["int"]
     f_facet        = format["facet"]
 
-    # Get data.
+    # Get data
     opt_par       = ir["optimise_parameters"]
     integral_type = ir["integral_type"]
     gdim          = ir["geometric_dimension"]
@@ -85,7 +82,7 @@ def _tabulate_tensor(ir, prefix, parameters):
     element_data  = ir["element_data"]
     num_cells     = ir["num_cells"]
 
-    # Create sets of used variables.
+    # Create sets of used variables
     used_weights    = set()
     used_psi_tables = set()
     used_nzcs       = set()
@@ -101,7 +98,8 @@ def _tabulate_tensor(ir, prefix, parameters):
         # Generate code for computing element tensor
         tensor_code, mem_code, num_ops = _generate_element_tensor(integrals,
                                                                   sets,
-                                                                  opt_par)
+                                                                  opt_par,
+                                                                  gdim)
         tensor_code = "\n".join(tensor_code)
 
         # Set operations equal to num_ops (for printing info on operations).
@@ -127,7 +125,7 @@ def _tabulate_tensor(ir, prefix, parameters):
         cases = [None for i in range(num_facets)]
         for i in range(num_facets):
             # Update transformer with facets and generate case code + set of used geometry terms.
-            c, mem_code, ops = _generate_element_tensor(integrals[i], sets, opt_par)
+            c, mem_code, ops = _generate_element_tensor(integrals[i], sets, opt_par, gdim)
             case = [f_comment("Total number of operations to compute element tensor (from this point): %d" % ops)]
             case += c
             cases[i] = "\n".join(case)
@@ -169,7 +167,8 @@ def _tabulate_tensor(ir, prefix, parameters):
                 # Update transformer with facets and generate case code + set of used geometry terms.
                 c, mem_code, ops = _generate_element_tensor(integrals[i][j],
                                                             sets,
-                                                            opt_par)
+                                                            opt_par,
+                                                            gdim)
                 case = [f_comment("Total number of operations to compute element tensor (from this point): %d" % ops)]
                 case += c
                 cases[i][j] = "\n".join(case)
@@ -200,7 +199,7 @@ def _tabulate_tensor(ir, prefix, parameters):
         jacobi_code += "\n\n" + format["generate cell volume"](tdim, gdim, integral_type)
         jacobi_code += "\n\n" + format["generate circumradius"](tdim, gdim, integral_type)
 
-    elif integral_type == "point":
+    elif integral_type == "vertex":
 
         # Iterate over vertices
         cases = [None for i in range(num_vertices)]
@@ -209,7 +208,8 @@ def _tabulate_tensor(ir, prefix, parameters):
             # set of used geometry terms.
             c, mem_code, ops = _generate_element_tensor(integrals[i],
                                                         sets,
-                                                        opt_par)
+                                                        opt_par,
+                                                        gdim)
             case = [f_comment("Total number of operations to compute element tensor (from this point): %d" % ops)]
             case += c
             cases[i] = "\n".join(case)
@@ -238,37 +238,48 @@ def _tabulate_tensor(ir, prefix, parameters):
         if num_cells > 2:
             warning("Custom integrals with more than two cells only partly supported.")
 
+        # Modify the dimensions of the primary indices because we have a macro element
+        if num_cells == 2:
+            prim_idims = [d*2 for d in prim_idims]
+
+        # Check whether we need to generate facet normals
+        generate_custom_facet_normal = num_cells == 2
+
         # Generate code for computing element tensor
         tensor_code, mem_code, num_ops = _generate_element_tensor(integrals,
                                                                   sets,
-                                                                  opt_par)
+                                                                  opt_par,
+                                                                  gdim,
+                                                                  generate_custom_facet_normal)
+
         tensor_code = "\n".join(tensor_code)
 
         # Set operations equal to num_ops (for printing info on operations).
         operations.append([num_ops])
 
+        # FIXME: Jacobi code is only needed when we use cell volume or circumradius.
+        # FIXME: Does not seem to be removed by removed_unused.
+
         # Generate code for basic geometric quantities
         jacobi_code = ""
         for i in range(num_cells):
+            r = i if num_cells > 1 else None
             jacobi_code += "\n"
             jacobi_code += f_comment("--- Compute geometric quantities on cell %d ---" % i)
             jacobi_code += "\n\n"
-            jacobi_code += f_comment("Extract vertex coordinates\n")
-            jacobi_code += format["extract_cell_coordinates"]((tdim + 1)*gdim*i, r=i)
-            jacobi_code += "\n\n"
-            jacobi_code += format["compute_jacobian"](tdim, gdim, r=i)
+            if num_cells > 1:
+                jacobi_code += f_comment("Extract vertex coordinates\n")
+                jacobi_code += format["extract_cell_coordinates"]((tdim + 1)*gdim*i, r=i)
+                jacobi_code += "\n\n"
+            jacobi_code += format["compute_jacobian"](tdim, gdim, r=r)
             jacobi_code += "\n"
-            jacobi_code += format["compute_jacobian_inverse"](tdim, gdim, r=i)
+            jacobi_code += format["compute_jacobian_inverse"](tdim, gdim, r=r)
             jacobi_code += "\n"
-            jacobi_code += format["generate cell volume"](tdim, gdim, integral_type, r=i)
+            jacobi_code += format["generate cell volume"](tdim, gdim, integral_type, r=r if num_cells > 1 else None)
             jacobi_code += "\n"
-            jacobi_code += format["generate circumradius"](tdim, gdim, integral_type, r=i)
+            jacobi_code += format["generate circumradius"](tdim, gdim, integral_type, r=r if num_cells > 1 else None)
             jacobi_code += "\n"
 
-        # FIXME: Jacobi code does not seem to be needed for custom
-        # integrals, so some of the above code can be removed
-        jacobi_code = ""
-
     else:
         error("Unhandled integral type: " + str(integral_type))
 
@@ -282,7 +293,7 @@ def _tabulate_tensor(ir, prefix, parameters):
 
     # Add common code except for custom integrals
     if integral_type != "custom":
-        common += _tabulate_weights([quadrature_weights[p] for p in used_weights])
+        common += _tabulate_weights([quadrature_weights[p] for p in sorted(used_weights)])
 
         # Add common code for updating tables
         name_map = ir["name_map"]
@@ -306,7 +317,7 @@ def _tabulate_tensor(ir, prefix, parameters):
     if prim_idims == []:
         common += [f_assign(f_A(f_int(0)), f_float(0))]
     else:
-        dim = functools.reduce(lambda v,u: v*u, prim_idims)
+        dim = functools.reduce(lambda v, u: v*u, prim_idims)
         common += f_loop([f_assign(f_A(f_r), f_float(0))], [(f_r, 0, dim)])
 
     # Create the constant geometry declarations (only generated if simplify expressions are enabled).
@@ -325,7 +336,7 @@ def _tabulate_tensor(ir, prefix, parameters):
     message = {"cell":           "Cell, number of operations to compute tensor: %s",
                "exterior_facet": "Exterior facet %d, number of operations to compute tensor: %s",
                "interior_facet": "Interior facets (%d, %d), number of operations to compute tensor: %s",
-               "point": "Point %s, number of operations to compute tensor: %s",
+               "vertex":         "Vertex %s, number of operations to compute tensor: %s",
                "custom":         "Custom domain, number of operations to compute tensor: %s"}
     for ops in operations:
         # Add geo ops count to integral ops count for writing info.
@@ -334,7 +345,7 @@ def _tabulate_tensor(ir, prefix, parameters):
 
     return "\n".join(common) + "\n" + tensor_code
 
-def _generate_element_tensor(integrals, sets, optimise_parameters):
+def _generate_element_tensor(integrals, sets, optimise_parameters, gdim, generate_custom_facet_normal=False):
     "Construct quadrature code for element tensors."
 
     # Prefetch formats to speed up code generation.
@@ -416,7 +427,6 @@ def _generate_element_tensor(integrals, sets, optimise_parameters):
 
         # Generate code to evaluate the element tensor.
         integral_code, ops = _generate_integral_code(points, terms, sets, optimise_parameters)
-
         num_ops += ops
         if points is None:
             quadrature_ops = "unknown"
@@ -428,6 +438,13 @@ def _generate_element_tensor(integrals, sets, optimise_parameters):
         element_code.append(f_comment\
             ("Number of operations to compute element tensor for following IP loop = %s" % str(quadrature_ops)))
 
+        # Generate code for custom facet normal if necessary
+        if generate_custom_facet_normal:
+            for line in ip_code:
+                if "n_00" in line:
+                    ip_code = [format["facet_normal_custom"](gdim)] + ip_code
+                    break
+
         # Loop code over all IPs.
         if points == 0:
             element_code.append(f_comment("Only 1 integration point, omitting IP loop."))
@@ -470,7 +487,7 @@ def _generate_functions(functions, sets):
 
     total_ops = 0
     # Loop ranges and get list of functions.
-    for loop_range, list_of_functions in function_list.items():
+    for loop_range, list_of_functions in sorted(function_list.items()):
         function_expr = {}
         function_numbers = []
         # Loop functions.
@@ -525,7 +542,7 @@ def _generate_integral_code(points, terms, sets, optimise_parameters):
     used_weights, used_psi_tables, used_nzcs, trans_set = sets
 
     # Loop terms and create code.
-    for loop, (data, entry_vals) in terms.items():
+    for loop, (data, entry_vals) in sorted(terms.items()):
 
         # If we don't have any entry values, there's no need to generate the
         # loop.
@@ -558,13 +575,12 @@ def _generate_integral_code(points, terms, sets, optimise_parameters):
 
             # Create comment for number of operations
             entry_ops_comment = f_comment("Number of operations to compute entry: %d" % entry_ops)
-
             entry_code = f_iadd(f_A(entry), value)
             loops[loop][0] += entry_ops
             loops[loop][1] += [entry_ops_comment, entry_code]
 
     # Write all the loops of basis functions.
-    for loop, ops_lines in loops.items():
+    for loop, ops_lines in sorted(loops.items()):
         ops, lines = ops_lines
         prim_ops = functools.reduce(lambda i, j: i*j, [ops] + [l[2] for l in loop])
         # Add number of operations for current loop to total count.
@@ -678,13 +694,13 @@ def _tabulate_psis(tables, used_psi_tables, inv_name_map, used_nzcs, optimise_pa
     # Get list of non zero columns, if we ignore ones, ignore columns with one component.
     if optimise_parameters["ignore ones"]:
         nzcs = []
-        for key, val in inv_name_map.items():
+        for key, val in sorted(inv_name_map.items()):
             # Check if we have a table of ones or if number of non-zero columns
             # is larger than one.
             if val[1] and len(val[1][1]) > 1 or not val[3]:
                 nzcs.append(val[1])
     else:
-        nzcs = [val[1] for key, val in inv_name_map.items()\
+        nzcs = [val[1] for key, val in sorted(inv_name_map.items())\
                                         if val[1]]
 
     # TODO: Do we get arrays that are not unique?
@@ -697,14 +713,14 @@ def _tabulate_psis(tables, used_psi_tables, inv_name_map, used_nzcs, optimise_pa
     # Construct name map.
     name_map = {}
     if inv_name_map:
-        for name in inv_name_map:
+        for name in sorted(inv_name_map):
             if inv_name_map[name][0] in name_map:
                 name_map[inv_name_map[name][0]].append(name)
             else:
                 name_map[inv_name_map[name][0]] = [name]
 
     # Loop items in table and tabulate.
-    for name in sorted(list(used_psi_tables)):
+    for name in sorted(used_psi_tables):
 
         # Only proceed if values are still used (if they're not remapped).
         vals = tables[name]
@@ -721,7 +737,7 @@ def _tabulate_psis(tables, used_psi_tables, inv_name_map, used_nzcs, optimise_pa
 
         # Tabulate non-zero indices.
         if optimise_parameters["eliminate zeros"]:
-            if name in name_map:
+            if name in sorted(name_map):
                 for n in name_map[name]:
                     if inv_name_map[n][1] and inv_name_map[n][1] in new_nzcs:
                         i, cols = inv_name_map[n][1]
@@ -736,7 +752,6 @@ def _tabulate_psis(tables, used_psi_tables, inv_name_map, used_nzcs, optimise_pa
                         new_nzcs.remove(inv_name_map[n][1])
     return code
 
-
 def _evaluate_basis_at_quadrature_points(psi_tables,
                                          gdim,
                                          element_data,
@@ -751,36 +766,55 @@ def _evaluate_basis_at_quadrature_points(psi_tables,
     f_static_array     = format["static array"]
     f_loop             = format["generate loop"]
     f_eval_basis_decl  = format["eval_basis_decl"]
+    f_eval_basis_init  = format["eval_basis_init"]
     f_eval_basis       = format["eval_basis"]
     f_eval_basis_copy  = format["eval_basis_copy"]
     f_eval_derivs_decl = format["eval_derivs_decl"]
+    f_eval_derivs_init = format["eval_derivs_init"]
     f_eval_derivs      = format["eval_derivs"]
     f_eval_derivs_copy = format["eval_derivs_copy"]
 
     code = []
 
     # Extract prefixes for tables
-    prefixes = sorted(list(set(table.split("_")[0] for table in psi_tables)))
+    prefixes = sorted(set(table.split("_")[0] for table in psi_tables))
 
     # Use lower case prefix for form name
     form_prefix = form_prefix.lower()
 
-    # For each uniqe prefix ("FE0" etc), figure out which derivatives
-    # need to be tabulated
-    used_derivatives = {}
+    # The psi_tables used by the quadrature code are for scalar
+    # components of specific derivatives, while tabulate_basis_all and
+    # tabulate_basis_derivatives_all return data including all
+    # possible components and derivatives. We therefore need to
+    # iterate over prefixes (= elements), call tabulate_basis_all or
+    # tabulate_basis_derivatives all, and then extract the relevant
+    # data and fill in the psi_tables. We therefore need to extract
+    # for each prefix, which tables need to be filled in.
+
+    # For each unique prefix, check which derivatives and components are used
+    used_derivatives_and_components = {}
     for prefix in prefixes:
-        derivatives = set()
+        used_derivatives_and_components[prefix] = {}
         for table in psi_tables:
-            if "_C" in table:
-                # FIXME: Bailout for now, add support for this later
-                error("custom integrals not yet supported for vector valued function spaces")
+            if not prefix in table: continue
+
+            # Check for derivative
             if "_D" in table:
                 d = table.split("_D")[1].split("_")[0]
-                n = sum([int(_d) for _d in d]) # FIXME: Will fail for more than 9 derivatives...
+                n = sum([int(_d) for _d in d]) # FIXME: Assume at most 9 derivatives...
             else:
                 n = 0
-            derivatives.add(n)
-        used_derivatives[prefix] = derivatives
+
+            # Check for component
+            if "_C" in table:
+                c = table.split("_C")[1].split("_")[0]
+            else:
+                c = None
+
+            # Note that derivative has been used
+            if not n in used_derivatives_and_components[prefix]:
+                used_derivatives_and_components[prefix][n] = set()
+            used_derivatives_and_components[prefix][n].add(c)
 
     # Generate code for setting quadrature weights
     code += [f_comment("Set quadrature weights")]
@@ -791,113 +825,183 @@ def _evaluate_basis_at_quadrature_points(psi_tables,
     for prefix in prefixes:
 
         # Get element data for current element
-        counter = int(prefix.split("FE")[1])
-        space_dim = element_data[counter]["local_dimension"]
-        value_size = element_data[counter]["value_size"]
-        macro_dim = space_dim*num_cells
+        counter        = int(prefix.split("FE")[1])
+        element_number = element_data[counter]["element_number"]
+        space_dim      = element_data[counter]["num_element_dofs"]
+        value_size     = element_data[counter]["value_size"]
 
         # Iterate over derivative orders
-        for n in used_derivatives[prefix]:
+        for n, components in sorted_by_key(used_derivatives_and_components[prefix]):
+            # components are a set and need to be sorted
+            components = sorted(components)
 
-            # Code for evaluate_basis_all
+            # Code for evaluate_basis_all (n = 0 which means it's not a derivative)
             if n == 0:
 
-                # Generate code for declaration of FE table
-                code += [f_comment("Create table %s for basis function values on all cells" % prefix)]
-                code += [f_eval_basis_decl % {"prefix": prefix, "macro_dim": macro_dim}]
+                code += [f_comment("--- Evaluation of basis functions ---")]
+                code += [""]
 
-                # Iterate over cells (in macro element)
-                for cell_number in range(num_cells):
+                # Compute variables for code generation
+                eval_stride = value_size
+                eval_size   = space_dim*eval_stride
+                table_size  = num_cells*space_dim
 
-                    code += [f_comment("--- Evaluation of basis functions on cell %d ---" % cell_number)]
+                # Iterate over components and initialize tables
+                for c in components:
+
+                    # Set name of table
+                    if c is None:
+                        table_name = prefix
+                    else:
+                        table_name = prefix + "_C%s" % c
+
+                    # Generate code for declaration of table
+                    code += [f_comment("Create table %s for basis function values on all cells" % table_name)]
+                    code += [f_eval_basis_decl % {"table_name": table_name}]
+                    code += [f_eval_basis_init % {"table_name": table_name,
+                                                  "table_size": table_size}]
                     code += [""]
 
-                    # Size of array to evaluate_basis_all
-                    num_vals = space_dim*value_size
+                # Iterate over cells in macro element and evaluate basis
+                for cell_number in range(num_cells):
 
-                    # Cell offset for array of vertex coordinates
+                    # Compute variables for code generation
+                    eval_name     = "%s_values_%d" % (prefix, cell_number)
+                    table_offset  = cell_number*space_dim
                     vertex_offset = cell_number*num_vertices*gdim
 
-                    # Cell offset for array of values
-                    values_offset = cell_number*num_vals
-
-                    # Name of values array
-                    values = "values_%d" % cell_number
-
                     # Generate block of code for loop
                     block = []
-                    block += [f_eval_basis      % {"prefix":        prefix,
-                                                   "form_prefix":   form_prefix,
-                                                   "gdim":          gdim,
-                                                   "counter":       counter,
-                                                   "vertex_offset": vertex_offset,
-                                                   "values":        values}]
-                    block += [f_eval_basis_copy % {"prefix":        prefix,
-                                                   "space_dim":     space_dim,
-                                                   "values_offset": values_offset,
-                                                   "values":        values}]
+
+                    # Generate code for calling evaluate_basis_all
+                    block += [f_eval_basis % {"form_prefix":    form_prefix,
+                                              "element_number": element_number,
+                                              "eval_name":      eval_name,
+                                              "gdim":           gdim,
+                                              "vertex_offset":  vertex_offset}]
+
+                    # Iterate over components and extract values
+                    for c in components:
+
+                        # Set name of table and component offset
+                        if c is None:
+                            table_name = prefix
+                            eval_offset = 0
+                        else:
+                            table_name = prefix + "_C%s" % c
+                            eval_offset = int(c)
+
+                        # Generate code for copying values
+                        block += [""]
+                        block += [f_eval_basis_copy % {"table_name":   table_name,
+                                                       "eval_name":    eval_name,
+                                                       "eval_stride":  eval_stride,
+                                                       "eval_offset":  eval_offset,
+                                                       "space_dim":    space_dim,
+                                                       "table_offset": table_offset}]
 
                     # Generate code
-                    code += [f_comment("Evaluate basis functions at all quadrature points")]
-                    code += [f_static_array("double", values, num_vals)]
+                    code += [f_comment("Evaluate basis functions on cell %d" % cell_number)]
+                    code += [f_static_array("double", eval_name, eval_size)]
                     code += f_loop(block, [("ip", 0, "num_quadrature_points")])
                     code += [""]
 
-            # Code for evaluate_basis_derivatives_all
+            # Code for evaluate_basis_derivatives_all (derivative of degree n > 0)
             else:
 
+                code += [f_comment("--- Evaluation of basis function derivatives of order %d ---" % n) ]
+                code += [""]
+
+                # FIXME: We extract values for all possible derivatives, even
+                # FIXME: if not all are used. (For components, we extract only
+                # FIXME: components that are actually used.) This may be optimized
+                # FIXME: but the extra cost is likely small.
+
                 # Get derivative tuples
-                deriv_tuples, _deriv_tuples = compute_derivative_tuples(n, gdim)
+                __, deriv_tuples = compute_derivative_tuples(n, gdim)
 
                 # Generate names for derivatives
-                derivs = ["".join(str(_d) for _d in d) for d in _deriv_tuples]
+                derivs = ["".join(str(_d) for _d in d) for d in deriv_tuples]
 
-                # Generate code for declaration of FE table
-                code += [f_comment("Create table(s) %s_D for order %d basis function derivatives on all cells" % (prefix, n))]
-                code += [(f_eval_derivs_decl % {"prefix": prefix, "d": d, "macro_dim": macro_dim}) for d in derivs]
+                # Compute variables for code generation
+                eval_stride = value_size*len(derivs)
+                eval_size   = space_dim*eval_stride
+                table_size  = num_cells*space_dim
+
+                # Iterate over derivatives and initialize tables
+                seen_derivs = set()
+                for d in derivs:
+
+                    # Skip derivative if seen before (d^2/dxdy = d^2/dydx)
+                    if d in seen_derivs: continue
+                    seen_derivs.add(d)
+
+                    # Iterate over components
+                    for c in components:
+
+                        # Set name of table
+                        if c is None:
+                            table_name = prefix + "_D%s" % d
+                        else:
+                            table_name = prefix + "_C%s_D%s" % (c, d)
+
+                        # Generate code for declaration of table
+                        code += [f_comment("Create table %s for basis function derivatives on all cells" % table_name)]
+                        code += [(f_eval_derivs_decl % {"table_name": table_name})]
+                        code += [(f_eval_derivs_init % {"table_name": table_name,
+                                                        "table_size": table_size})]
+                        code += [""]
 
                 # Iterate over cells (in macro element)
                 for cell_number in range(num_cells):
 
-                    code += [f_comment("--- Evaluation of basis function derivatives on cell %d ---" % cell_number)]
-                    code += [""]
-
-                    # Size of array to evaluate_basis_derivatives_all
-                    num_vals = space_dim*value_size*len(derivs)
-
-                    # Cell offset for array of vertex coordinates
+                    # Compute variables for code generation
+                    eval_name     = "%s_dvalues_%d_%d" % (prefix, n, cell_number)
+                    table_offset  = cell_number*space_dim
                     vertex_offset = cell_number*num_vertices*gdim
 
-                    # Cell offset for array of values
-                    values_offset = cell_number*num_vals
-
-                    # Stride for values array
-                    stride = value_size*len(derivs)
-
-                    # Name of values array
-                    values = "dvalues_%d_%d" % (n, cell_number)
-
                     # Generate block of code for loop
                     block = []
-                    block += [f_eval_derivs %       {"prefix":        prefix,
-                                                     "form_prefix":   form_prefix,
-                                                     "gdim":          gdim,
-                                                     "n":             n,
-                                                     "counter":       counter,
-                                                     "vertex_offset": vertex_offset,
-                                                     "values":        values}]
-                    block += [(f_eval_derivs_copy % {"prefix":        prefix,
-                                                     "space_dim":     space_dim,
-                                                     "values_offset": values_offset,
-                                                     "d":             d,
-                                                     "offset":        i,
-                                                     "stride":        stride,
-                                                     "values":        values
-                                                     }) for (i, d) in enumerate(derivs)]
+
+                    # Generate code for calling evaluate_basis_derivatives_all
+                    block += [f_eval_derivs % {"form_prefix":    form_prefix,
+                                               "element_number": element_number,
+                                               "eval_name":      eval_name,
+                                               "gdim":           gdim,
+                                               "vertex_offset":  vertex_offset,
+                                               "n":              n}]
+
+                    # Iterate over derivatives and extract values
+                    seen_derivs = set()
+                    for i, d in enumerate(derivs):
+
+                        # Skip derivative if seen before (d^2/dxdy = d^2/dydx)
+                        if d in seen_derivs: continue
+                        seen_derivs.add(d)
+
+                        # Iterate over components
+                        for c in components:
+
+                            # Set name of table and component offset
+                            if c is None:
+                                table_name = prefix + "_D%s" % d
+                                eval_offset = i
+                            else:
+                                table_name = prefix + "_C%s_D%s" % (c, d)
+                                eval_offset = len(derivs)*int(c) + i
+
+                            # Generate code for copying values
+                            block += [""]
+                            block += [(f_eval_derivs_copy % {"table_name":   table_name,
+                                                             "eval_name":    eval_name,
+                                                             "eval_stride":  eval_stride,
+                                                             "eval_offset":  eval_offset,
+                                                             "space_dim":    space_dim,
+                                                             "table_offset": table_offset})]
 
                     # Generate code
-                    code += [f_comment("Evaluate basis function derivatives at all quadrature points")]
-                    code += [f_static_array("double", values, num_vals)]
+                    code += [f_comment("Evaluate basis function derivatives on cell %d" % cell_number)]
+                    code += [f_static_array("double", eval_name, eval_size)]
                     code += f_loop(block, [("ip", 0, "num_quadrature_points")])
                     code += [""]
 
