@@ -20,9 +20,10 @@ transformers to translate UFL expressions."""
 #
 # Modified by Martin Alnaes, 2013
 # Modified by Garth N. Wells, 2013
+# Modified by Lizao Li, 2015
 #
 # First added:  2009-10-13
-# Last changed: 2014-10-08
+# Last changed: 2015-03-20
 
 # Python modules.
 from six.moves import zip
@@ -928,9 +929,16 @@ class QuadratureTransformerBase(Transformer):
 
         # Get subelement and the relative (flattened) component (in case we have mixed elements).
         local_comp, local_elem = ufl_element.extract_component(component)
-        ffc_assert(len(local_comp) <= 1, "Assuming there are no tensor-valued basic elements.")
-        local_comp = local_comp[0] if local_comp else 0
 
+        # For basic tensor elements, local_comp should be flattened
+        if len(local_comp) and len(local_elem.value_shape()) > 0:
+            # Map component using component map from UFL. (TODO: inefficient use of this function)
+            comp_map, _ = build_component_numbering(local_elem.value_shape(), local_elem.symmetry())
+            local_comp = comp_map[local_comp]
+            
+        # Set local_comp to 0 if it is ()
+        if not local_comp: local_comp = 0
+           
         # Check that component != not () since the UFL component map will turn
         # it into 0, and () does not mean zeroth component in this context.
         if len(component):
