@@ -12,7 +12,7 @@ function "foo", one should only need to use the data stored
 in the intermediate representation under the key "foo".
 """
 
-# Copyright (C) 2009-2014 Anders Logg
+# Copyright (C) 2009-2015 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -31,15 +31,14 @@ in the intermediate representation under the key "foo".
 #
 # Modified by Marie E. Rognes 2010
 # Modified by Kristian B. Oelgaard 2010
-# Modified by Martin Alnaes, 2013-2014
+# Modified by Martin Alnaes, 2013-2015
+# Modified by Lizao Li 2015
 
 # Python modules
 from itertools import chain
 
 # Import UFL
 import ufl
-from ufl.classes import Measure
-from ufl.cell import cellname2dim
 
 # FFC modules
 from ffc.utils import compute_permutations, product
@@ -47,9 +46,13 @@ from ffc.log import info, error, begin, end, debug_ir, ffc_assert, warning
 from ffc.fiatinterface import create_element, reference_cell
 from ffc.mixedelement import MixedElement
 from ffc.enrichedelement import EnrichedElement, SpaceOfReals
+from ffc.fiatinterface import DiscontinuousLagrangeTrace
 from ffc.quadratureelement import QuadratureElement
 from ffc.cpp import set_float_formatting
 
+# List of supported integral types
+ufc_integral_types = ["cell", "exterior_facet", "interior_facet", "vertex", "custom"]
+
 def pick_representation(representation):
     "Return one of the specialized code generation modules from a representation string."
     if representation == "quadrature":
@@ -154,7 +157,7 @@ def _compute_dofmap_ir(ufl_element, element_id, element_numbers):
     ir["topological_dimension"] = domain.topological_dimension()
     ir["geometric_dimension"] = domain.geometric_dimension()
     ir["global_dimension"] = _global_dimension(element)
-    ir["local_dimension"] = element.space_dimension()
+    ir["num_element_dofs"] = element.space_dimension()
     ir["num_facet_dofs"] = len(facet_dofs[0])
     ir["num_entity_dofs"] = num_dofs_per_entity
     ir["tabulate_dofs"] = _tabulate_dofs(element, cell)
@@ -242,9 +245,8 @@ def _compute_form_ir(form_data, form_id, element_numbers):
     ir["create_finite_element"] = [element_numbers[e] for e in form_data.elements]
     ir["create_dofmap"] = [element_numbers[e] for e in form_data.elements]
 
-    integral_types = ["cell", "exterior_facet", "interior_facet", "point", "custom"]
-    for integral_type in integral_types:
-        ir["num_%s_domains" % integral_type] = _num_foo_domains(integral_type, form_data)
+    for integral_type in ufc_integral_types:
+        ir["max_%s_subdomain_id" % integral_type] = _max_foo_subdomain_id(integral_type, form_data)
         ir["has_%s_integrals" % integral_type] = _has_foo_integrals(integral_type, form_data)
         ir["create_%s_integral" % integral_type] = _create_foo_integral(integral_type, form_data)
         ir["create_default_%s_integral" % integral_type] = _create_default_foo_integral(integral_type, form_data)
@@ -330,7 +332,7 @@ def _evaluate_dof(ufl_element, element):
             "physical_value_size": _value_size(ufl_element),
             "geometric_dimension": domain.geometric_dimension(),
             "topological_dimension": domain.topological_dimension(),
-            "dofs": [L.pt_dict for L in element.dual_basis()],
+            "dofs": [L.pt_dict if L else None for L in element.dual_basis()],
             "physical_offsets": _generate_physical_offsets(ufl_element)}
 
 def _extract_elements(element):
@@ -367,7 +369,7 @@ def _evaluate_basis(ufl_element, element):
 
     # This function is evidently not implemented for TensorElements
     for e in elements:
-        if len(e.value_shape()) > 1:
+        if (len(e.value_shape()) > 1) and (e.num_sub_elements() != 1):
             return "Function not supported/implemented for TensorElements."
 
     # Handle QuadratureElement, not supported because the basis is only defined
@@ -375,6 +377,8 @@ def _evaluate_basis(ufl_element, element):
     for e in elements:
         if isinstance(e, QuadratureElement):
             return "Function not supported/implemented for QuadratureElement."
+        if isinstance(e, DiscontinuousLagrangeTrace):
+            return "Function not supported for Trace elements"
 
     # Initialise data with 'global' values.
     data = {"reference_value_size": _value_size(element),
@@ -396,11 +400,17 @@ def _evaluate_basis(ufl_element, element):
             coefficients = []
             coeffs = e.get_coeffs()
 
-            # Handle coefficients for vector valued basis elements
-            # [Raviart-Thomas, Brezzi-Douglas-Marini (BDM)].
-            if num_components > 1:
+            if (num_components > 1) and (len(e.value_shape()) == 1):
+                # Handle coefficients for vector valued basis elements
+                # [Raviart-Thomas, Brezzi-Douglas-Marini (BDM)].
                 for c in range(num_components):
                     coefficients.append(coeffs[i][c])
+            elif (num_components > 1) and (len(e.value_shape()) == 2):
+                # Handle coefficients for tensor valued basis elements.
+                # [Regge]
+                for p in range(e.value_shape()[0]):
+                    for q in range(e.value_shape()[1]):
+                        coefficients.append(coeffs[i][p][q])
             else:
                 coefficients.append(coeffs[i])
 
@@ -424,7 +434,7 @@ def _evaluate_basis(ufl_element, element):
 def _tabulate_coordinates(ufl_element, element):
     "Compute intermediate representation of tabulate_coordinates."
 
-    if uses_integral_moments(element):
+    if uses_integral_moments(element) or not element.dual_basis()[0]:
         return {}
 
     domain, = ufl_element.domains() # Assuming single domain
@@ -504,6 +514,8 @@ def _interpolate_vertex_values(ufl_element, element):
     for e in all_elements(element):
         if isinstance(e, QuadratureElement):
             return "Function is not supported/implemented for QuadratureElement."
+        if isinstance(e, DiscontinuousLagrangeTrace):
+            return "Function is not implemented for DiscontinuousLagrangeTrace."
 
     domain, = ufl_element.domains() # Assuming single domain
     cellname = domain.cell().cellname()
@@ -514,7 +526,7 @@ def _interpolate_vertex_values(ufl_element, element):
 
     # Check whether computing the Jacobian is necessary
     mappings = element.mapping()
-    ir["needs_jacobian"] = any("piola" in m for m in mappings)
+    ir["needs_jacobian"] = any("piola" in m for m in mappings) or any("pullback as metric" in m for m in mappings)
     ir["needs_oriented"] = needs_oriented_jacobian(element)
 
     # See note in _evaluate_dofs
@@ -559,9 +571,9 @@ def _create_foo_integral(integral_type, form_data):
     return [itg_data.subdomain_id for itg_data in form_data.integral_data
            if itg_data.integral_type == integral_type and isinstance(itg_data.subdomain_id, int)]
 
-def _num_foo_domains(integral_type, form_data):
-    "Compute intermediate representation of num_foo_domains."
-    return form_data.num_sub_domains.get(integral_type, 0)
+def _max_foo_subdomain_id(integral_type, form_data):
+    "Compute intermediate representation of max_foo_subdomain_id."
+    return form_data.num_sub_domains.get(integral_type, 0) # TODO: Rename in form_data
 
 def _has_foo_integrals(integral_type, form_data):
     "Compute intermediate representation of has_foo_integrals."
@@ -656,10 +668,11 @@ def uses_integral_moments(element):
     "True if element uses integral moments for its degrees of freedom."
 
     integrals = set(["IntegralMoment", "FrobeniusIntegralMoment"])
-    tags = set([L.get_type_tag() for L in element.dual_basis()])
+    tags = set([L.get_type_tag() for L in element.dual_basis() if L])
     return len(integrals & tags) > 0
 
 def needs_oriented_jacobian(element):
-    # Check whether this element needs an oriented jacobian (only
-    # contravariant piolas seem to need it)
-    return ("contravariant piola" in element.mapping())
+    # Check whether this element needs an oriented jacobian
+    # (only contravariant piolas and pullback as metric seem to need it)
+    return ("contravariant piola" in element.mapping() or
+            "pullback as metric" in element.mapping())
