@@ -1,7 +1,7 @@
 """This module contains utility functions for some code shared between
 quadrature and tensor representation."""
 
-# Copyright (C) 2012-2014 Marie Rognes
+# Copyright (C) 2012-2015 Marie Rognes
 #
 # This file is part of FFC.
 #
@@ -18,7 +18,7 @@ quadrature and tensor representation."""
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Martin Alnaes 2013
+# Modified by Martin Alnaes 2013-2015
 # Modified by Anders Logg 2014
 
 from ufl.measure import integral_type_to_measure_name
@@ -105,8 +105,13 @@ def initialize_integral_ir(representation, itg_data, form_data, form_id):
     entity_type = {"cell":           "cell",
                    "exterior_facet": "facet",
                    "interior_facet": "facet",
-                   "point":          "vertex",
-                   "custom":         "cell"}[itg_data.integral_type]
+                   "vertex":         "vertex",
+                   #"point":          "vertex", # TODO: Not sure, clarify here what 'entity_type' refers to?
+                   "custom":         "cell",
+                   #"overlap":        "cell",
+                   #"interface":      "cell", # FIXME: set this to the same as "custom" but not sure here...
+                   #"cutcell":        "cell",
+                   }[itg_data.integral_type]
 
     # Extract data
     cell = itg_data.domain.cell()
@@ -145,6 +150,7 @@ def generate_enabled_coefficients(enabled_coefficients):
 def initialize_integral_code(ir, prefix, parameters):
     "Representation independent default initialization of code dict for integral from intermediate representation."
     code = {}
+    code["class_type"] = ir["integral_type"] + "_integral"
     code["restrict"] = parameters["restrict_keyword"]
     code["classname"] = format["classname " + ir["integral_type"] + "_integral"](prefix, ir["form_id"], ir["subdomain_id"])
     code["members"] = ""
