@@ -16,11 +16,10 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Marie Rognes, 2010.
-#
-# Last changed: 2010-04-07
 
-from ffc.log import error
 import numpy
+from ufl.utils.sorting import sorted_by_key
+from ffc.log import error
 
 class RestrictedElement:
     "Create a restriction of a given FIAT element."
@@ -57,7 +56,7 @@ class RestrictedElement:
     def tabulate(self, order, points):
         result = self._element.tabulate(order, points)
         extracted = {}
-        for (dtuple, values) in result.iteritems():
+        for (dtuple, values) in sorted_by_key(result):
             extracted[dtuple] = numpy.array([values[i] for i in self._indices])
         return extracted
 
@@ -79,9 +78,9 @@ def _extract_entity_dofs(element, indices):
     # FIXME: Readability counts
     entity_dofs = element.entity_dofs()
     dofs = {}
-    for (dim, entities) in entity_dofs.iteritems():
+    for (dim, entities) in sorted_by_key(entity_dofs):
         dofs[dim] = {}
-        for (entity, all_dofs) in entities.iteritems():
+        for (entity, all_dofs) in sorted_by_key(entities):
             dofs[dim][entity] = []
             for index in all_dofs:
                 if index in indices:
