@@ -1,4 +1,4 @@
-# Copyright (C) 2013 Martin Alnaes
+# Copyright (C) 2013-2014 Martin Alnaes
 #
 # This file is part of FFC.
 #
@@ -14,21 +14,21 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
-#
-# First added:  2013-02-09
-# Last changed: 2013-03-04
 
-#import numpy
 from ffc.log import info, error, begin, end, debug_ir, ffc_assert, warning
-#from ffc.cpp import format
+
 from ffc.fiatinterface import create_element
 from ffc.representationutils import initialize_integral_ir
-#from ffc.quadrature.quadratureutils import create_psi_tables
-from ffc.quadrature.quadraturerepresentation import _parse_optimise_parameters, _sort_integrals, _tabulate_basis
+from ffc.quadrature.parameters import parse_optimise_parameters
+from ffc.quadrature.tabulate_basis import tabulate_basis
+from ffc.quadrature.quadraturerepresentation import sort_integrals
+
+from uflacs.backends.ffc.representation import compute_uflacs_integral_ir
 
 def compute_integral_ir(itg_data,
                         form_data,
                         form_id,
+                        element_numbers, # FIXME: Not used, what's this for?
                         parameters):
     "Compute intermediate represention of integral."
 
@@ -37,18 +37,27 @@ def compute_integral_ir(itg_data,
     # Initialise representation
     ir = initialize_integral_ir("uflacs", itg_data, form_data, form_id)
 
-    # Sort integrals into a dict with number of integral points as key
-    sorted_integrals = _sort_integrals(itg_data.integrals, itg_data.metadata, form_data)
+    # Sort integrals into a dict with quadrature degree and rule as key
+    sorted_integrals = sort_integrals(itg_data.integrals,
+                                      itg_data.metadata["quadrature_degree"],
+                                      itg_data.metadata["quadrature_rule"])
 
+    # TODO: Might want to create the uflacs ir first and then create the tables we need afterwards!
     # Tabulate quadrature points and basis function values in these points
-    integrals_dict, psi_tables, quad_weights = \
-        _tabulate_basis(sorted_integrals, itg_data.domain_type, form_data)
+    integrals_dict, psi_tables, quadrature_rules = \
+        tabulate_basis(sorted_integrals, form_data, itg_data)
+
+    # Delegate to flacs to build its intermediate representation and add to ir
+    uflacs_ir = compute_uflacs_integral_ir(psi_tables, ir["entitytype"], integrals_dict, form_data, parameters)
+
+    # Store uflacs generated part separately
+    ir["uflacs"] = uflacs_ir
+
+    # Create and save the optisation parameters # TODO: Define uflacs specific optimization parameters instead
+    #ir["optimise_parameters"] = parse_optimise_parameters(parameters)
 
     # Save tables for quadrature weights and points
-    ir["quadrature_weights"] = quad_weights
-
-    # Save tables for basis function values
-    ir["psi_tables"] = psi_tables
+    ir["quadrature_rules"] = quadrature_rules
 
     # Create dimensions of primary indices, needed to reset the argument 'A'
     # given to tabulate_tensor() by the assembler.
@@ -59,13 +68,4 @@ def compute_integral_ir(itg_data,
     ir["coeff_idims"] = [create_element(ufl_element).space_dimension()
                          for ufl_element in form_data.coefficient_elements]
 
-    # Create and save the optisation parameters.
-    ir["optimise_parameters"] = _parse_optimise_parameters(parameters)
-
-    # Delegate to flacs to build its intermediate representation and add to ir
-    import uflacs.backends.ffc
-    uir = uflacs.backends.ffc.compute_tabulate_tensor_ir(ir, integrals_dict,
-                                                         form_data, parameters)
-    ir.update(uir)
-
     return ir
