@@ -18,6 +18,7 @@
 # First added:  2010-01-18
 # Last changed: 2010-01-18
 
+from __future__ import print_function
 from cppcode import evaluate_basis_derivatives_code
 from ufl import FiniteElement, MixedElement
 from instant.output import get_status_output
@@ -93,19 +94,19 @@ def check_results(values, reference):
     missing_refs = []
     diffs = []
     num_ok = 0
-    print ""
-    for element, deriv_orders in values.items():
-        print "\nResults for %s:" % element
+    print("")
+    for element, deriv_orders in list(values.items()):
+        print("\nResults for %s:" % element)
 
-        for deriv_order, vals in deriv_orders.items():
+        for deriv_order, vals in list(deriv_orders.items()):
             if vals is None:
-                print "Error"
+                print("Error")
                 continue
 
             # Get reference values
             if not element in reference or not deriv_order in reference[element]:
                 missing_refs.append(element + "order %d" % deriv_order)
-                print "Missing reference"
+                print("Missing reference")
                 continue
 
             refs = reference[element][deriv_order]
@@ -114,24 +115,24 @@ def check_results(values, reference):
             e = max(abs(vals - refs))
             if e < tol:
                 num_ok += 1
-                print "Derivative order: %d, OK: (diff = %g)" % (deriv_order, e)
+                print("Derivative order: %d, OK: (diff = %g)" % (deriv_order, e))
             else:
-                print "*** (Derivative order: %d, diff = %g)" % (deriv_order, e)
+                print("*** (Derivative order: %d, diff = %g)" % (deriv_order, e))
                 diffs.append(element + "order %d" % deriv_order)
 
     if ffc_failed == gcc_failed == run_failed == missing_refs == diffs:
-        print "\nAll %d elements verified OK" % len(reference)
+        print("\nAll %d elements verified OK" % len(reference))
         return 0
     if len(ffc_failed) > 0:
-        print "\n*** FFC compilation failed for the following elements:\n" + "\n".join(ffc_failed)
+        print("\n*** FFC compilation failed for the following elements:\n" + "\n".join(ffc_failed))
     if len(gcc_failed) > 0:
-        print "\n*** g++ compilation failed for the following elements:\n" + "\n".join(gcc_failed)
+        print("\n*** g++ compilation failed for the following elements:\n" + "\n".join(gcc_failed))
     if len(run_failed) > 0:
-        print "\n*** Evaluation failed (seg. fault?) for the following elements:\n" + "\n".join(run_failed)
+        print("\n*** Evaluation failed (seg. fault?) for the following elements:\n" + "\n".join(run_failed))
     if len(missing_refs) > 0:
-        print "\n*** No reference values were found for the following elements:\n" + "\n".join(missing_refs)
+        print("\n*** No reference values were found for the following elements:\n" + "\n".join(missing_refs))
     if len(diffs) > 0:
-        print "\n*** Difference in values were found for the following elements:\n" + "\n".join(diffs)
+        print("\n*** Difference in values were found for the following elements:\n" + "\n".join(diffs))
     return 1
 
 def compile_element(ufl_element):
@@ -198,12 +199,12 @@ def compute_values(ufl_element, deriv_order):
 def print_refs():
     if os.path.isfile("reference.pickle"):
         reference = pickle.load(open("reference.pickle", "r"))
-        for elem, derivs in reference.items():
-            for deriv_order, vals in derivs.items():
-                print
-                print elem
-                print deriv_order
-                print vals
+        for elem, derivs in list(reference.items()):
+            for deriv_order, vals in list(derivs.items()):
+                print()
+                print(elem)
+                print(deriv_order)
+                print(vals)
     else:
         raise RuntimeError("No references to print")
 
@@ -221,28 +222,28 @@ def main(args):
 
     values = {}
     # Evaluate basis for single elements
-    print "\nComputing evaluate_basis_derivatives for single elements"
+    print("\nComputing evaluate_basis_derivatives for single elements")
     for element in single_elements:
         for shape in element["shapes"]:
             for order in element["orders"]:
                 ufl_element = FiniteElement(element["family"], shape, order)
-                print "Compiling element: ", str(ufl_element)
+                print("Compiling element: ", str(ufl_element))
                 error = compile_element(ufl_element)
                 if error:
                     continue
-                print "Computing values"
+                print("Computing values")
                 values[repr(ufl_element)] = {}
                 for deriv_order in range(1,4):
                     values[repr(ufl_element)][deriv_order] = compute_values(ufl_element, deriv_order)
 
     # Evaluate basis for single elements
-    print "\nComputing evaluate_basis_derivatives for mixed elements"
+    print("\nComputing evaluate_basis_derivatives for mixed elements")
     for ufl_element in mixed_elements:
-        print "Compiling element: ", str(ufl_element)
+        print("Compiling element: ", str(ufl_element))
         error = compile_element(ufl_element)
         if error:
             continue
-        print "Computing values"
+        print("Computing values")
         values[repr(ufl_element)] = {}
         for deriv_order in range(1,4):
             values[repr(ufl_element)][deriv_order] = compute_values(ufl_element, deriv_order)
@@ -252,7 +253,7 @@ def main(args):
     if os.path.isfile("reference.pickle"):
         reference = pickle.load(open("reference.pickle", "r"))
     else:
-        print "Unable to find reference values, storing current values."
+        print("Unable to find reference values, storing current values.")
         pickle.dump(values, open("reference.pickle", "w"))
         return 0
 
