@@ -18,11 +18,12 @@
 # First added:  2010-01-18
 # Last changed: 2010-01-29
 
-from cppcode import evaluate_basis_code
+from __future__ import print_function
+from .cppcode import evaluate_basis_code
 from ufl import FiniteElement, MixedElement
 from instant.output import get_status_output
 import sys, os, pickle, numpy, shutil
-from elements import single_elements, mixed_elements
+from .elements import single_elements, mixed_elements
 
 ffc_failed = []
 gcc_failed = []
@@ -33,8 +34,8 @@ def check_results(values, reference):
 
     # Check if we have missing values.
     missing_vals = []
-    num_refs = len(reference.keys())
-    for element in reference.keys():
+    num_refs = len(list(reference.keys()))
+    for element in list(reference.keys()):
         if not element in values:
             missing_vals.append(element)
 
@@ -42,20 +43,20 @@ def check_results(values, reference):
     diffs = []
     correct = []
     num_ok = 0
-    print ""
+    print("")
     sorted_elements = sorted(values.keys())
     for element in sorted_elements:
         vals = values[element]
-        print "\nResults for %s:" % element
+        print("\nResults for %s:" % element)
 
         if vals is None:
-            print "Error"
+            print("Error")
             continue
 
         # Get reference values
         if not element in reference:
             missing_refs.append(element)
-            print "Missing reference"
+            print("Missing reference")
             continue
         refs = reference[element]
         tol = 1e-12
@@ -63,30 +64,30 @@ def check_results(values, reference):
         e = max(abs(vals - refs))
         if e < tol:
             num_ok += 1
-            print "OK: (diff = %g)" % e
+            print("OK: (diff = %g)" % e)
             correct.append(element)
         else:
-            print "*** (diff = %g)" % e
+            print("*** (diff = %g)" % e)
             diffs.append(element)
 
     if ffc_failed == gcc_failed == run_failed == missing_refs == diffs == missing_vals:
-        print "\nAll %d elements verified OK" % len(reference)
+        print("\nAll %d elements verified OK" % len(reference))
         return 0
     else:
-        print "\n*** The values were correct for the following elements:\n" + "\n\n".join(correct)
+        print("\n*** The values were correct for the following elements:\n" + "\n\n".join(correct))
     if len(ffc_failed) > 0:
-        print "\n*** FFC compilation failed for the following elements:\n" + "\n\n".join(ffc_failed)
+        print("\n*** FFC compilation failed for the following elements:\n" + "\n\n".join(ffc_failed))
     if len(gcc_failed) > 0:
-        print "\n*** g++ compilation failed for the following elements:\n" + "\n\n".join(gcc_failed)
+        print("\n*** g++ compilation failed for the following elements:\n" + "\n\n".join(gcc_failed))
     if len(run_failed) > 0:
-        print "\n*** Evaluation failed (seg. fault?) for the following elements:\n" + "\n\n".join(run_failed)
+        print("\n*** Evaluation failed (seg. fault?) for the following elements:\n" + "\n\n".join(run_failed))
     if len(missing_refs) > 0:
-        print "\n*** No reference values were found for the following elements:\n" + "\n\n".join(missing_refs)
+        print("\n*** No reference values were found for the following elements:\n" + "\n\n".join(missing_refs))
     if len(missing_vals) > 0:
-        print "\n*** No values were computed the following %d elements:\n" % len(missing_vals) +\
-              "\n\n".join(missing_vals)
+        print("\n*** No values were computed the following %d elements:\n" % len(missing_vals) +\
+              "\n\n".join(missing_vals))
     if len(diffs) > 0:
-        print "\n*** Difference in values were found for the following elements:\n" + "\n\n".join(diffs)
+        print("\n*** Difference in values were found for the following elements:\n" + "\n\n".join(diffs))
 
     num_ffc = len(ffc_failed)
     num_gcc = len(gcc_failed)
@@ -95,15 +96,15 @@ def check_results(values, reference):
     num_val = len(missing_vals)
     num_cor = len(correct)
     num_dif = len(diffs)
-    print "\nNum ref elements: ", num_refs
-    print "Num ffc fail:     ", num_ffc
-    print "Num gcc fail:     ", num_gcc
-    print "Num run fail:     ", num_run
-    print "Num miss ref:     ", num_ref
-    print "Num miss val:     ", num_val
-    print "Num correct:      ", num_cor
-    print "Num diff:         ", num_dif
-    print "Total:            ", num_ffc + num_gcc + num_run + num_ref + num_val + num_cor + num_dif
+    print("\nNum ref elements: ", num_refs)
+    print("Num ffc fail:     ", num_ffc)
+    print("Num gcc fail:     ", num_gcc)
+    print("Num run fail:     ", num_run)
+    print("Num miss ref:     ", num_ref)
+    print("Num miss val:     ", num_val)
+    print("Num correct:      ", num_cor)
+    print("Num diff:         ", num_dif)
+    print("Total:            ", num_ffc + num_gcc + num_run + num_ref + num_val + num_cor + num_dif)
 
     return 1
 
@@ -169,10 +170,10 @@ def compute_values(ufl_element):
 def print_refs():
     if os.path.isfile("reference.pickle"):
         reference = pickle.load(open("reference.pickle", "r"))
-        for elem, vals in reference.items():
-            print
-            print elem
-            print vals
+        for elem, vals in list(reference.items()):
+            print()
+            print(elem)
+            print(vals)
     else:
         raise RuntimeError("No references to print")
 
@@ -190,28 +191,28 @@ def main(args):
 
     values = {}
     # Evaluate basis for single elements
-    print "\nComputing evaluate_basis for single elements"
+    print("\nComputing evaluate_basis for single elements")
     for element in single_elements:
         for shape in element["shapes"]:
             for order in element["orders"]:
                 ufl_element = FiniteElement(element["family"], shape, order)
-                print "Compiling element: ", str(ufl_element)
+                print("Compiling element: ", str(ufl_element))
                 error = compile_element(ufl_element)
                 if error:
                     values[repr(ufl_element)] = None
                     continue
-                print "Computing values"
+                print("Computing values")
                 values[repr(ufl_element)] = compute_values(ufl_element)
 
     # Evaluate basis for mixed elements
-    print "\nComputing evaluate_basis for mixed elements"
+    print("\nComputing evaluate_basis for mixed elements")
     for ufl_element in mixed_elements:
-        print "Compiling element: ", str(ufl_element)
+        print("Compiling element: ", str(ufl_element))
         error = compile_element(ufl_element)
         if error:
             values[repr(ufl_element)] = None
             continue
-        print "Computing values"
+        print("Computing values")
         values[repr(ufl_element)] = compute_values(ufl_element)
 
     # Load or update reference values
@@ -219,7 +220,7 @@ def main(args):
     if os.path.isfile("reference.pickle"):
         reference = pickle.load(open("reference.pickle", "r"))
     else:
-        print "Unable to find reference values, storing current values."
+        print("Unable to find reference values, storing current values.")
         pickle.dump(values, open("reference.pickle", "w"))
         return 0
 
