@@ -21,7 +21,7 @@
 #
 # First added:  2007-02-06
 # Last changed: 2009-02-24
-
+from __future__ import print_function
 import unittest
 import sys
 import numpy
@@ -86,10 +86,10 @@ class FunctionValueTests(unittest.TestCase):
     def _check_function_values(self, points, element, reference):
         for x in points:
             table = element.tabulate(0, (x,))
-            basis = table[table.keys()[0]]
+            basis = table[list(table.keys())[0]]
             for i in range(len(basis)):
                 if element.value_shape() == ():
-                    self.assertAlmostEqual(basis[i], reference[i](x))
+                    self.assertAlmostEqual(float(basis[i]), reference[i](x))
                 else:
                     for k in range(element.value_shape()[0]):
                         self.assertAlmostEqual(basis[i][k][0],
@@ -211,6 +211,39 @@ class FunctionValueTests(unittest.TestCase):
                                  6*x[1]- 3*x[0]*x[1] - 6*x[1]**2),
                       ]
 
+    def testDRT1_2D(self):
+        "Test values of DRT1."
+
+        element = create(FiniteElement("Discontinuous Raviart-Thomas", "triangle", 1))
+        reference = [lambda x: (x[0], x[1]),
+                     lambda x: (1 - x[0], -x[1]),
+                     lambda x: (x[0], x[1] - 1)]
+
+        points = [random_point(triangle) for i in range(num_points)]
+        self._check_function_values(points, element, reference)
+
+    def testDRT2_2D(self):
+        "Test values of DRT2."
+
+        element = create(FiniteElement("Discontinuous Raviart-Thomas", "triangle", 2))
+
+        reference = [ lambda x: (-x[0] + 3*x[0]**2,
+                                 -x[1] + 3*x[0]*x[1]),
+                      lambda x: (-x[0] + 3*x[0]*x[1],
+                                 -x[1] + 3*x[1]**2),
+                      lambda x: ( 2 - 5*x[0] - 3*x[1] + 3*x[0]*x[1] + 3*x[0]**2,
+                                  -2*x[1] + 3*x[0]*x[1] + 3*x[1]**2),
+                      lambda x: (-1.0 + x[0] + 3*x[1] - 3*x[0]*x[1],
+                                 x[1] - 3*x[1]**2),
+                      lambda x: (2*x[0] - 3*x[0]*x[1] - 3*x[0]**2,
+                                 -2 + 3*x[0]+ 5*x[1] - 3*x[0]*x[1] - 3*x[1]**2),
+                      lambda x: (- x[0] + 3*x[0]**2,
+                                 + 1 - 3*x[0] - x[1] + 3*x[0]*x[1]),
+                      lambda x: (6*x[0] - 3*x[0]*x[1] - 6*x[0]**2,
+                                 3*x[1] - 6*x[0]*x[1] - 3*x[1]**2),
+                      lambda x: (3*x[0] - 6*x[0]*x[1] - 3*x[0]**2,
+                                 6*x[1]- 3*x[0]*x[1] - 6*x[1]**2),
+                      ]
 
         points = [random_point(triangle) for i in range(num_points)]
         self._check_function_values(points, element, reference)
@@ -237,6 +270,16 @@ class FunctionValueTests(unittest.TestCase):
         points = [random_point(tetrahedron) for i in range(num_points)]
         self._check_function_values(points, element, reference)
 
+    def testDRT1_3D(self):
+        element = create(FiniteElement("DRT", "tetrahedron", 1))
+        reference = [lambda x: (-x[0], -x[1], -x[2]),
+                     lambda x: (-1.0 + x[0], x[1], x[2]),
+                     lambda x: (-x[0], 1.0 - x[1], -x[2]),
+                     lambda x: ( x[0], x[1], -1.0 + x[2])
+                     ]
+        points = [random_point(tetrahedron) for i in range(num_points)]
+        self._check_function_values(points, element, reference)
+
     def testBDM1_3D(self):
         element = create(FiniteElement("BDM", "tetrahedron", 1))
         reference = [ lambda x: (-3*x[0], x[1], x[2]),
@@ -297,7 +340,7 @@ class JITTests(unittest.TestCase):
         jit(a0, options)
         dt0 = time() - t
 
-        print ""
+        print("")
 
         # Compile a1 (should be fairly fast, using disk cache)
         t = time()
@@ -308,10 +351,10 @@ class JITTests(unittest.TestCase):
         dt0_good = 0.005
         dt1_good = 0.01
 
-        print ""
-        print "JIT in-memory cache:", dt0
-        print "JIT disk cache:     ", dt1
-        print "Reasonable values are %g and %g" % (dt0_good, dt1_good)
+        print("")
+        print("JIT in-memory cache:", dt0)
+        print("JIT disk cache:     ", dt1)
+        print("Reasonable values are %g and %g" % (dt0_good, dt1_good))
 
         # Check times
         self.assertTrue(dt0 < 10*dt0_good)
