@@ -11,11 +11,11 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2010-01-06
 # Last changed: 2010-02-01
@@ -206,11 +206,11 @@ class TestReduceGIP(unittest.TestCase):
                    ])
 
 #        print "\nReduceGIP"
-#        start = time.time()
+        start = time.time()
         expr_exp = expr.expand()
 #        print "ReduceGIP: time, expand()      ", time.time() - start
 
-#        start = time.time()
+        start = time.time()
         expr_red = expr_exp.reduce_ops()
 #        print "ReduceGIP: time, reduce_ops(): ", time.time() - start
 
@@ -230,7 +230,7 @@ class TestReduceGIP(unittest.TestCase):
         self.assertAlmostEqual(eval(str(expr)), eval(str(expr_red)))
         self.assertEqual(expr.ops(), 314)
         self.assertEqual(expr_exp.ops(), 314)
-        self.assertEqual(expr_red.ops(), 114)
+        self.assertEqual(expr_red.ops(), 120)
 
 if __name__ == "__main__":
 
