@@ -1,7 +1,7 @@
 // This file provides utility functions for computing geometric quantities.
 // This code is released into the public domain.
 //
-// The FEniCS Project (http://www.fenicsproject.org/) 2013.
+// The FEniCS Project (http://www.fenicsproject.org/) 2013-2015.
 
 #ifndef __UFC_GEOMETRY_H
 #define __UFC_GEOMETRY_H
@@ -93,6 +93,100 @@ static const double tetrahedron_vertices[UFC_NUM_VERTICES_IN_TETRAHEDRON][UFC_TD
   {0.0, 0.0, 1.0}
   };
 
+// FIXME: Insert quad conventions here
+/*
+static const double quadrilateral_vertices[UFC_NUM_VERTICES_IN_QUADRILATERAL][UFC_TDIM_2] = {
+  {0.0, 0.0},
+  {0.0, 0.0},
+  {0.0, 0.0},
+  {0.0, 0.0},
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const double hexahedron_vertices[UFC_NUM_VERTICES_IN_HEXAHEDRON][UFC_TDIM_3] = {
+  {0.0, 0.0, 0.0},
+  {0.0, 0.0, 0.0},
+  {0.0, 0.0, 0.0},
+  {0.0, 0.0, 0.0},
+  {0.0, 0.0, 0.0},
+  {0.0, 0.0, 0.0},
+  {0.0, 0.0, 0.0},
+  {0.0, 0.0, 0.0},
+  };
+*/
+
+/// --- Local reference cell barycenter by UFC conventions ---
+
+static const double interval_barycenter[UFC_TDIM_1] = {
+  0.5
+  };
+
+static const double triangle_barycenter[UFC_TDIM_2] = {
+  1.0/3.0, 1.0/3.0
+  };
+
+static const double tetrahedron_barycenter[UFC_TDIM_3] = {
+  0.25, 0.25, 0.25
+  };
+
+// FIXME: Insert quad conventions here
+/*
+static const double quadrilateral_barycenter[UFC_TDIM_2] = {
+  0.5, 0.5
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const double hexahedron_barycenter[UFC_TDIM_3] = {
+  0.5, 0.5, 0.5
+  };
+*/
+
+/// --- Local reference cell facet barycenters by UFC conventions ---
+
+static const double interval_facet_barycenter[UFC_NUM_FACETS_IN_INTERVAL][UFC_TDIM_1] = {
+  {0.0},
+  {1.0}
+  };
+
+static const double triangle_facet_barycenter[UFC_NUM_FACETS_IN_TRIANGLE][UFC_TDIM_2] = {
+  {0.5, 0.5},
+  {0.0, 0.5},
+  {0.5, 0.0}
+  };
+
+static const double tetrahedron_facet_barycenter[UFC_NUM_FACETS_IN_TETRAHEDRON][UFC_TDIM_3] = {
+  {0.5, 0.5, 0.5},
+  {0.0, 1.0/3.0, 1.0/3.0},
+  {1.0/3.0, 0.0, 1.0/3.0},
+  {1.0/3.0, 1.0/3.0, 0.0},
+  };
+
+// FIXME: Insert quad conventions here
+/*
+static const double quadrilateral_facet_barycenter[UFC_NUM_FACETS_IN_QUADRILATERAL][UFC_TDIM_2] = {
+  {0.0, 0.0},
+  {0.0, 0.0},
+  {0.0, 0.0},
+  {0.0, 0.0},
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const double hexahedron_facet_barycenter[UFC_NUM_FACETS_IN_HEXAHEDRON][UFC_TDIM_3] = {
+  {0.0, 0.5, 0.5},
+  {0.0, 0.5, 0.5},
+  {0.0, 0.5, 0.5},
+  {0.0, 0.5, 0.5},
+  {0.0, 0.5, 0.5},
+  {0.0, 0.5, 0.5},
+  };
+*/
+
 /// --- Local reference cell facet orientations by UFC conventions ---
 
 static const double interval_facet_orientations[UFC_NUM_FACETS_IN_INTERVAL] = {
@@ -113,6 +207,28 @@ static const double tetrahedron_facet_orientations[UFC_NUM_FACETS_IN_TETRAHEDRON
   -1.0
   };
 
+// FIXME: Insert quad conventions here
+/*
+static const double quadrilateral_facet_orientations[UFC_NUM_FACETS_IN_QUADRILATERAL] = {
+  +1.0,
+  +1.0,
+  +1.0,
+  +1.0,
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const double hexahedron_facet_orientations[UFC_NUM_FACETS_IN_HEXAHEDRON] = {
+  +1.0,
+  +1.0,
+  +1.0,
+  +1.0,
+  +1.0,
+  +1.0,
+  };
+*/
+
 /// --- Local reference cell entity relations by UFC conventions ---
 
 static const unsigned int triangle_edge_vertices[UFC_NUM_EDGES_IN_TRIANGLE][2] = {
@@ -130,6 +246,34 @@ static const unsigned int tetrahedron_edge_vertices[UFC_NUM_EDGES_IN_TETRAHEDRON
   {0, 1}
   };
 
+// FIXME: Insert quad conventions here
+/*
+static const unsigned int quadrilateral_edge_vertices[UFC_NUM_EDGES_IN_QUADRILATERAL][2] = {
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const unsigned int hexahedron_edge_vertices[UFC_NUM_EDGES_IN_HEXAHEDRON][2] = {
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  };
+*/
+
 /// --- Local reference cell entity relations by UFC conventions ---
 
 static const unsigned int interval_facet_vertices[UFC_NUM_FACETS_IN_INTERVAL][1] = {
@@ -157,6 +301,40 @@ static const unsigned int tetrahedron_facet_edge_vertices[UFC_NUM_FACETS_IN_TETR
   {{1, 2}, {0, 2}, {0, 1}}
   };
 
+// FIXME: Insert quad conventions here
+/*
+static const unsigned int quadrilateral_facet_vertices[UFC_NUM_FACETS_IN_QUADRILATERAL][UFC_NUM_VERTICES_IN_INTERVAL] = {
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  {0, 0},
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const unsigned int hexahedron_facet_vertices[UFC_NUM_FACETS_IN_HEXAHEDRON][UFC_NUM_VERTICES_IN_QUADRILATERAL] = {
+  {0, 0, 0, 0},
+  {0, 0, 0, 0},
+  {0, 0, 0, 0},
+  {0, 0, 0, 0},
+  {0, 0, 0, 0},
+  {0, 0, 0, 0},
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const unsigned int hexahedron_facet_edge_vertices[UFC_NUM_FACETS_IN_HEXAHEDRON][UFC_NUM_FACETS_IN_QUADRILATERAL][UFC_NUM_VERTICES_IN_INTERVAL] = {
+  {{0, 0}, {0, 0}, {0, 0}, {0, 0}},
+  {{0, 0}, {0, 0}, {0, 0}, {0, 0}},
+  {{0, 0}, {0, 0}, {0, 0}, {0, 0}},
+  {{0, 0}, {0, 0}, {0, 0}, {0, 0}},
+  {{0, 0}, {0, 0}, {0, 0}, {0, 0}},
+  {{0, 0}, {0, 0}, {0, 0}, {0, 0}},
+  };
+*/
+
 /// --- Reference cell edge vectors by UFC conventions (edge vertex 1 - edge vertex 0 for each edge in cell) ---
 
 static const double triangle_reference_edge_vectors[UFC_NUM_EDGES_IN_TRIANGLE][UFC_TDIM_2] = {
@@ -198,6 +376,119 @@ static const double tetrahedron_facet_reference_edge_vectors[UFC_NUM_FACETS_IN_T
   },
   };
 
+// FIXME: Insert quad conventions here
+/*
+static const double quadrilateral_reference_edge_vectors[UFC_NUM_EDGES_IN_QUADRILATERAL][UFC_TDIM_2] = {
+  { 0.0, 0.0},
+  { 0.0, 0.0},
+  { 0.0, 0.0},
+  { 0.0, 0.0},
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const double hexahedron_reference_edge_vectors[UFC_NUM_EDGES_IN_HEXAHEDRON][UFC_TDIM_3] = {
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+// Edge vectors for each quadrilateral facet of a hexahedron
+static const double hexahedron_facet_reference_edge_vectors[UFC_NUM_FACETS_IN_HEXAHEDRON][UFC_NUM_EDGES_IN_QUADRILATERAL][UFC_TDIM_3] = {
+  { // facet 0
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+  },
+  { // facet 1
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+  },
+  { // facet 2
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+  },
+  { // facet 3
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+  },
+  { // facet 4
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+  },
+  { // facet 5
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+    { 0.0,  0.0,  0.0},
+  },
+  };
+*/
+
+/// --- Reference cell facet normals by UFC conventions (outwards pointing on reference cell) ---
+
+static const double interval_reference_facet_normals[UFC_NUM_FACETS_IN_INTERVAL][UFC_TDIM_1] = {
+  {-1.0},
+  {+1.0},
+  };
+
+static const double triangle_reference_facet_normals[UFC_NUM_FACETS_IN_TRIANGLE][UFC_TDIM_2] = {
+  { 0.7071067811865476, 0.7071067811865476 },
+  {-1.0,  0.0},
+  { 0.0, -1.0},
+  };
+
+static const double tetrahedron_reference_facet_normals[UFC_NUM_FACETS_IN_TETRAHEDRON][UFC_TDIM_3] = {
+  {0.5773502691896258, 0.5773502691896258, 0.5773502691896258},
+  {-1.0,  0.0,  0.0},
+  { 0.0, -1.0,  0.0},
+  { 0.0,  0.0, -1.0},
+  };
+
+// FIXME: Insert quad conventions here
+/*
+static const double quadrilateral_reference_facet_normals[UFC_NUM_FACETS_IN_QUADRILATERAL][UFC_TDIM_2] = {
+  { 0.0,  0.0 },
+  { 0.0,  0.0 },
+  { 0.0,  0.0 },
+  { 0.0,  0.0 },
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const double hexahedron_reference_facet_normals[UFC_NUM_FACETS_IN_HEXAHEDRON][UFC_TDIM_3] = {
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  { 0.0,  0.0,  0.0},
+  };
+*/
+
 /// --- Jacobians of reference facet cell to reference cell coordinate mappings by UFC conventions ---
 
 static const double triangle_reference_facet_jacobian[UFC_NUM_FACETS_IN_TRIANGLE][UFC_TDIM_2][UFC_TDIM_2-1] = {
@@ -213,6 +504,28 @@ static const double tetrahedron_reference_facet_jacobian[UFC_NUM_FACETS_IN_TETRA
   { { 1.0,  0.0}, {0.0, 1.0}, {0.0, 0.0} },
   };
 
+// FIXME: Insert quad conventions here
+/*
+static const double quadrilateral_reference_facet_jacobian[UFC_NUM_FACETS_IN_QUADRILATERAL][UFC_TDIM_2][UFC_TDIM_2-1] = {
+  { { 0.0}, { 0.0} },
+  { { 0.0}, { 0.0} },
+  { { 0.0}, { 0.0} },
+  { { 0.0}, { 0.0} },
+  };
+*/
+
+// FIXME: Insert quad conventions here
+/*
+static const double hexahedron_reference_facet_jacobian[UFC_NUM_FACETS_IN_HEXAHEDRON][UFC_TDIM_3][UFC_TDIM_3-1] = {
+  { { 0.0,  0.0}, {0.0, 0.0}, {0.0, 0.0} },
+  { { 0.0,  0.0}, {0.0, 0.0}, {0.0, 0.0} },
+  { { 0.0,  0.0}, {0.0, 0.0}, {0.0, 0.0} },
+  { { 0.0,  0.0}, {0.0, 0.0}, {0.0, 0.0} },
+  { { 0.0,  0.0}, {0.0, 0.0}, {0.0, 0.0} },
+  { { 0.0,  0.0}, {0.0, 0.0}, {0.0, 0.0} },
+  };
+*/
+
 /// --- Coordinate mappings from reference facet cell to reference cell by UFC conventions ---
 
 inline void compute_reference_facet_to_reference_cell_coordinates_interval(double Xc[UFC_TDIM_1], unsigned int facet)
